@@ -1,8 +1,12 @@
-from foxes.core import Algorithm, FarmDataModelList
+import numpy as np
+
+from foxes.core import Algorithm, FarmDataModelList, get_engine
 from foxes.core import PointDataModel, PointDataModelList, FarmController
+from foxes.config import config
 import foxes.models as fm
 import foxes.variables as FV
 import foxes.constants as FC
+
 from . import models as mdls
 
 
@@ -28,6 +32,8 @@ class Downwind(Algorithm):
     partial_wakes: dict
         The partial wakes mapping. Key: wake model name,
         value: foxes.core.PartialWakesModel
+    deflection: foxes.core.WakeDeflection
+        The wake deflection model
     ground_models: dict
         The ground models mapping. Key: wake model name,
         value: foxes.core.GroundModel
@@ -49,6 +55,7 @@ class Downwind(Algorithm):
         FV.AMB_REWS,
         FV.AMB_TI,
         FV.AMB_RHO,
+        FV.AMB_CT,
         FV.AMB_P,
         FV.WD,
         FV.REWS,
@@ -56,8 +63,6 @@ class Downwind(Algorithm):
         FV.TI,
         FV.CT,
         FV.P,
-        FV.ORDER,
-        FV.WEIGHT,
     ]
 
     def __init__(
@@ -67,13 +72,12 @@ class Downwind(Algorithm):
         wake_models,
         rotor_model="centre",
         wake_frame="rotor_wd",
+        wake_deflection="no_deflection",
         partial_wakes=None,
         ground_models=None,
         farm_controller="basic_ctrl",
-        chunks={FC.STATE: 1000, FC.POINT: 4000},
         mbook=None,
-        dbook=None,
-        verbosity=1,
+        **kwargs,
     ):
         """
         Constructor.
@@ -93,6 +97,8 @@ class Downwind(Algorithm):
         wake_frame: str
             The wake frame. Will be looked up in the
             model book
+        deflection: foxes.core.WakeDeflection, optional
+            The wake deflection model
         partial_wakes: dict, list or str, optional
             The partial wakes mapping. Key: wake model name,
             value: partial wake model name
@@ -102,35 +108,32 @@ class Downwind(Algorithm):
         farm_controller: str
             The farm controller. Will be
             looked up in the model book
-        chunks: dict
-            The chunks choice for running in parallel with dask,
-            e.g. `{"state": 1000}` for chunks of 1000 states
         mbook: foxes.ModelBook, optional
             The model book
-        dbook: foxes.DataBook, optional
-            The data book, or None for default
-        verbosity: int
-            The verbosity level, 0 means silent
+        kwargs: dict, optional
+            Additional parameters for the base class
 
         """
         if mbook is None:
             mbook = fm.ModelBook()
 
-        super().__init__(mbook, farm, chunks, verbosity, dbook)
+        super().__init__(mbook, farm, **kwargs)
 
-        self.states = states
+        self.__states = states
         self.n_states = None
-        self.states_data = None
 
-        self.rotor_model = self.mbook.rotor_models[rotor_model]
+        self.__rotor_model = self.mbook.rotor_models.get_item(rotor_model)
         self.rotor_model.name = rotor_model
 
-        self.wake_frame = self.mbook.wake_frames[wake_frame]
+        self.__wake_frame = self.mbook.wake_frames.get_item(wake_frame)
         self.wake_frame.name = wake_frame
 
-        self.wake_models = {}
+        self.__wake_deflection = self.mbook.wake_deflections.get_item(wake_deflection)
+        self.wake_deflection.name = wake_deflection
+
+        self.__wake_models = {}
         for w in wake_models:
-            m = self.mbook.wake_models[w]
+            m = self.mbook.wake_models.get_item(w)
             m.name = w
             self.wake_models[w] = m
 
@@ -144,10 +147,12 @@ class Downwind(Algorithm):
                     try:
                         pw = values
                         if checkw:
-                            mbooks[pw].check_wmodel(self.wake_models[w], error=True)
+                            mbooks.get_item(pw).check_wmodel(
+                                self.wake_models[w], error=True
+                            )
                     except TypeError:
                         pw = deffunc(self.wake_models[w])
-                    target[w] = mbooks[pw]
+                    target[w] = mbooks.get_item(pw)
                     target[w].name = pw
             elif isinstance(values, list):
                 for i, w in enumerate(wake_models):
@@ -156,7 +161,7 @@ class Downwind(Algorithm):
                             f"Not enough {descr} in list {values}, expecting {len(wake_models)}"
                         )
                     pw = values[i]
-                    target[w] = mbooks[pw]
+                    target[w] = mbooks.get_item(pw)
                     target[w].name = pw
             else:
                 for w in wake_models:
@@ -164,10 +169,10 @@ class Downwind(Algorithm):
                         pw = values[w]
                     else:
                         pw = deffunc(self.wake_models[w])
-                    target[w] = mbooks[pw]
+                    target[w] = mbooks.get_item(pw)
                     target[w].name = pw
 
-        self.partial_wakes = {}
+        self.__partial_wakes = {}
         _set_wspecific(
             descr="partial wakes",
             target=self.partial_wakes,
@@ -177,7 +182,7 @@ class Downwind(Algorithm):
             checkw=True,
         )
 
-        self.ground_models = {}
+        self.__ground_models = {}
         _set_wspecific(
             descr="ground models",
             target=self.ground_models,
@@ -187,8 +192,126 @@ class Downwind(Algorithm):
             checkw=False,
         )
 
-        self.farm_controller = self.mbook.farm_controllers[farm_controller]
+        self.__farm_controller = self.mbook.farm_controllers.get_item(farm_controller)
         self.farm_controller.name = farm_controller
+        self.farm_controller.find_turbine_types(self)
+
+    @property
+    def states(self):
+        """
+        The states
+
+        Returns
+        -------
+        m: foxes.core.States
+            The states
+
+        """
+        return self.__states
+
+    @states.setter
+    def states(self, value):
+        """Resets the states"""
+        if self.running:
+            raise ValueError(f"{self.name}: Cannot set states while running")
+        if self.states.initialized:
+            self.states.finalize(self, verbosity=self.verbosity)
+        self.__states = value
+        self.init_states()
+
+    @property
+    def rotor_model(self):
+        """
+        The rotor model
+
+        Returns
+        -------
+        m: foxes.core.RotorModel
+            The rotor model
+
+        """
+        return self.__rotor_model
+
+    @property
+    def wake_models(self):
+        """
+        The wake models
+
+        Returns
+        -------
+        m: dict
+            The wake models. Key: name,
+            value: foxes.core.WakeModel
+
+        """
+        return self.__wake_models
+
+    @property
+    def wake_frame(self):
+        """
+        The wake frame
+
+        Returns
+        -------
+        m: foxes.core.WakeFrame
+            The wake frame
+
+        """
+        return self.__wake_frame
+
+    @property
+    def wake_deflection(self):
+        """
+        The wake deflection
+
+        Returns
+        -------
+        m: foxes.core.WakeDeflection
+            The wake deflection model
+
+        """
+        return self.__wake_deflection
+
+    @property
+    def partial_wakes(self):
+        """
+        The partial wakes models
+
+        Returns
+        -------
+        m: dict
+            The partial wakes models. Key: name,
+            value: foxes.core.PartialWakesModel
+
+        """
+        return self.__partial_wakes
+
+    @property
+    def ground_models(self):
+        """
+        The ground models
+
+        Returns
+        -------
+        m: dict
+            The ground models, key: name,
+            value: foxes.core.GroundModel
+
+        """
+        return self.__ground_models
+
+    @property
+    def farm_controller(self):
+        """
+        The farm controller
+
+        Returns
+        -------
+        m: foxes.core.FarmController
+            The farm controller
+
+        """
+        return self.__farm_controller
 
     @classmethod
     def get_model(cls, name):
@@ -208,39 +331,59 @@ class Downwind(Algorithm):
         """
         return getattr(mdls, name)
 
-    def _print_deco(self, func_name, n_points=None):
+    def update_n_turbines(self):
+        """
+        Reset the number of turbines,
+        according to self.farm
+        """
+        if self.n_turbines != self.farm.n_turbines:
+            super().update_n_turbines()
+            self.farm_controller.find_turbine_types(self)
+
+    def print_deco(self, func_name=None, n_points=None):
         """
         Helper function for printing model names
+
+        Parameters
+        ----------
+        func_name: str, optional
+            Name of the calling function
+        n_points: int, optional
+            The number of points
+
         """
         if self.verbosity > 0:
-            deco = "-" * 50
+            deco = "-" * 60
             print(f"\n{deco}")
-            print(f"  Running {self.name}: {func_name}")
+            print(f"  Algorithm: {type(self).__name__}")
+            if func_name is not None:
+                print(f"  Running {self.name}: {func_name}")
             print(deco)
             print(f"  n_states : {self.n_states}")
             print(f"  n_turbines: {self.n_turbines}")
             if n_points is not None:
                 print(f"  n_points : {n_points}")
             print(deco)
-            print(f"  states   : {self.states}")
-            print(f"  rotor    : {self.rotor_model}")
+            print(f"  states    : {self.states}")
+            print(f"  rotor     : {self.rotor_model}")
             print(f"  controller: {self.farm_controller}")
             print(f"  wake frame: {self.wake_frame}")
+            print(f"  deflection: {self.wake_deflection}")
             print(deco)
-            print(f"  wakes:")
+            print("  wakes:")
             for i, w in enumerate(self.wake_models.values()):
                 print(f"    {i}) {w.name}: {w}")
             print(deco)
-            print(f"  partial wakes:")
+            print("  partial wakes:")
             for i, (w, p) in enumerate(self.partial_wakes.items()):
                 print(f"    {i}) {w}: {p.name}, {p}")
             print(deco)
-            print(f"  turbine models:")
+            print("  turbine models:")
             for i, m in enumerate(self.farm_controller.pre_rotor_models.models):
                 print(f"    {i}) {m.name}: {m} [pre-rotor]")
             for i, m in enumerate(self.farm_controller.post_rotor_models.models):
                 print(
-                    f"    {i+len(self.farm_controller.pre_rotor_models.models)}) {m.name}: {m}"
+                    f"    {i + len(self.farm_controller.pre_rotor_models.models)}) {m.name}: {m}"
                 )
             print(deco)
             print()
@@ -252,7 +395,7 @@ class Downwind(Algorithm):
         if self.verbosity > 0:
             deco = "-" * 50
             print(f"\n{deco}")
-            print(f"  Model oder")
+            print("  Model oder")
             print(f"{deco}")
 
             for i, m in enumerate(mlist.models):
@@ -276,29 +419,26 @@ class Downwind(Algorithm):
             self.states.initialize(self, self.verbosity)
             self.n_states = self.states.size()
 
-    def all_models(self, with_states=True):
+    def sub_models(self):
         """
-        Return all models
-
-        Parameters
-        ----------
-        with_states: bool
-            Flag for including states
+        List of all sub-models
 
         Returns
         -------
-        mdls: list of foxes.core.Model
-            The list of models
+        smdls: list of foxes.core.Model
+            All sub models
 
         """
-        mdls = [self.states] if with_states else []
-        mdls += [
-            self.rotor_model,
+        mdls = [
+            self.states,
             self.farm_controller,
+            self.rotor_model,
+            self.wake_deflection,
             self.wake_frame,
         ]
         mdls += list(self.wake_models.values())
         mdls += list(self.partial_wakes.values())
+        mdls += list(self.ground_models.values())
 
         return mdls
 
@@ -306,13 +446,10 @@ class Downwind(Algorithm):
         """
         Initializes the algorithm.
         """
-        self.print(f"\nInitializing algorithm '{self.name}'")
-        super().initialize()
-
         self.init_states()
 
-        for m in self.all_models(with_states=False):
-            m.initialize(self, self.verbosity)
+        self.print(f"\nInitializing algorithm '{self.name}'")
+        super().initialize()
 
     def _collect_farm_models(
         self,
@@ -340,9 +477,7 @@ class Downwind(Algorithm):
         # 2) calculate ambient rotor results:
         mlist.models.append(self.rotor_model)
         calc_pars.append(calc_parameters.get(mlist.models[-1].name, {}))
-        calc_pars[-1].update(
-            {"store_rpoints": True, "store_rweights": True, "store_amb_res": True}
-        )
+        calc_pars[-1]["store"] = True
 
         # 3) run post-rotor turbine models via farm controller:
         mlist.models.append(self.farm_controller)
@@ -360,7 +495,7 @@ class Downwind(Algorithm):
             calc_pars.append(calc_parameters.get(mlist.models[-1].name, {}))
 
         # 6) reorder back to state-turbine dimensions:
-        if outputs != False:
+        if not isinstance(outputs, bool) or outputs:
             mlist.models.append(self.get_model("ReorderFarmOutput")(outputs))
             calc_pars.append(calc_parameters.get(mlist.models[-1].name, {}))
 
@@ -368,19 +503,46 @@ class Downwind(Algorithm):
 
     def _calc_farm_vars(self, mlist):
         """Helper function that gathers the farm variables"""
-        self.farm_vars = sorted(list(set([FV.WEIGHT] + mlist.output_farm_vars(self))))
+        self.farm_vars = sorted(list(mlist.output_farm_vars(self)))
 
-    def _run_farm_calc(self, mlist, *data, outputs=None, **kwargs):
-        """Helper function for running the main farm calculation"""
-        self.print(
-            f"\nCalculating {self.n_states} states for {self.n_turbines} turbines"
-        )
+    def _launch_parallel_farm_calc(
+        self,
+        mlist,
+        model_data,
+        outputs=None,
+        normalize=False,
+        **kwargs,
+    ):
+        """
+        Runs the main calculation, launching parallelization
+
+        Parameters
+        ----------
+        mlist: foxes.models.FarmDataModelList
+            The model list
+        model_data: xarray.Dataset
+            The initial model data
+        outputs: list of str, optional
+            The output variables, or None for defaults
+        normalize: bool
+            Normalize the weights to 1 wrt sum over states
+        kwargs: dict, optional
+            Additional parameters for running
+
+        Returns
+        -------
+        farm_results: xarray.Dataset
+            The farm results. The calculated variables have
+            dimensions (state, turbine)
+
+        """
         out_vars = self.farm_vars if outputs is None else outputs
-        farm_results = mlist.run_calculation(self, *data, out_vars=out_vars, **kwargs)
-        farm_results[FC.TNAME] = ((FC.TURBINE,), self.farm.turbine_names)
-        for v in [FV.ORDER, FV.ORDER_SSEL, FV.ORDER_INV]:
-            if v in farm_results:
-                farm_results[v] = farm_results[v].astype(FC.ITYPE)
+        farm_results = get_engine().run_calculation(
+            self, mlist, model_data, out_vars=out_vars, **kwargs
+        )
+
+        if normalize:
+            farm_results[FV.WEIGHT] /= farm_results[FV.WEIGHT].sum(dim=FC.STATE)
 
         return farm_results
 
@@ -388,10 +550,8 @@ class Downwind(Algorithm):
         self,
         outputs=None,
         calc_parameters={},
-        persist=True,
         finalize=True,
         ambient=False,
-        chunked_results=False,
         **kwargs,
     ):
         """
@@ -404,15 +564,10 @@ class Downwind(Algorithm):
             Key: model name str, value: parameter dict
         outputs: list of str, optional
             The output variables, or None for defaults
-        persist: bool
-            Switch for forcing dask to load all model data
-            into memory
         finalize: bool
             Flag for finalization after calculation
         ambient: bool
             Flag for ambient instead of waked calculation
-        chunked_results: bool
-            Flag for chunked results
         kwargs: dict, optional
             Additional parameters for run_calculation
 
@@ -428,7 +583,7 @@ class Downwind(Algorithm):
             self.initialize()
 
         # welcome:
-        self._print_deco("calc_farm")
+        self.print_deco("calc_farm")
 
         # collect models:
         if outputs == "default":
@@ -437,7 +592,7 @@ class Downwind(Algorithm):
 
         # initialize models:
         if not mlist.initialized:
-            mlist.initialize(self, self.verbosity)
+            mlist.initialize(self, self.verbosity - 1)
             self._calc_farm_vars(mlist)
         self._print_model_oder(mlist, calc_pars)
 
@@ -448,37 +603,37 @@ class Downwind(Algorithm):
             outputs = sorted(list(set(outputs).intersection(self.farm_vars)))
 
         # get input model data:
-        models_data = self.get_models_data()
-        if persist:
-            models_data = models_data.persist()
-        self.print("\nInput data:\n\n", models_data, "\n")
-        self.print(f"\nFarm variables:", ", ".join(self.farm_vars))
-        self.print(f"\nOutput variables:", ", ".join(outputs))
-        self.print(f"\nChunks: {self.chunks}\n")
+        model_data = self.get_models_data()
+        self.print("\nInput data:\n\n", model_data, "\n")
+        self.print("\nFarm variables:", ", ".join(self.farm_vars))
+        self.print("\nOutput variables:", ", ".join(outputs))
 
         # run main calculation:
-        farm_results = self._run_farm_calc(
+        farm_results = super().calc_farm(
             mlist,
-            models_data,
+            model_data,
             parameters=calc_pars,
             outputs=outputs,
             **kwargs,
         )
-        del models_data
+        farm_results[FC.TNAME] = ((FC.TURBINE,), self.farm.turbine_names)
+        for v in [FV.ORDER, FV.ORDER_SSEL, FV.ORDER_INV]:
+            if v in farm_results:
+                farm_results[v] = farm_results[v].astype(config.dtype_int)
+        del model_data
 
         # finalize models:
         if finalize:
             self.print("\n")
-            mlist.finalize(self, self.verbosity)
+            mlist.finalize(self, self.verbosity - 1)
             self.finalize()
+        else:
+            self.del_model_data(mlist)
 
         if ambient:
             dvars = [v for v in farm_results.data_vars.keys() if v in FV.var2amb]
             farm_results = farm_results.drop_vars(dvars)
 
-        if chunked_results:
-            farm_results = self.chunked(farm_results)
-
         return farm_results
 
     def _collect_point_models(
@@ -534,17 +689,49 @@ class Downwind(Algorithm):
 
         return mlist, calc_pars
 
+    def _launch_parallel_points_calc(self, mlist, *data, outputs=None, **kwargs):
+        """
+        Runs the main points calculation, launching parallelization
+
+        Parameters
+        ----------
+        mlist: foxes.models.FarmDataModelList
+            The model list
+        data: tuple of xarray.Dataset
+            The (mdata, fdata) inputs
+        outputs: list of str, optional
+            The output variables, or None for defaults
+        kwargs: dict, optional
+            Additional parameters for running
+
+        Returns
+        -------
+        point_results: xarray.Dataset
+            The point results. The calculated variables have
+            dimensions (state, point)
+
+        """
+        return (
+            get_engine()
+            .run_calculation(self, mlist, *data, out_vars=outputs, **kwargs)
+            .sel({FC.TPOINT: 0})
+            .rename({FC.TARGET: FC.POINT})
+        )
+
     def calc_points(
         self,
         farm_results,
         points,
         point_models=None,
+        outputs=None,
         calc_parameters={},
         persist_mdata=True,
         persist_pdata=False,
         finalize=True,
         ambient=False,
         chunked_results=False,
+        states_sel=None,
+        states_isel=None,
         **kwargs,
     ):
         """
@@ -557,6 +744,8 @@ class Downwind(Algorithm):
             dimensions (state, turbine)
         points: numpy.ndarray
             The points of interest, shape: (n_states, n_points, 3)
+        outputs: list of str, optional
+            The output variables, or None for defaults
         point_models: str or foxes.core.PointDataModel
             Additional point models to be executed
         calc_parameters: dict
@@ -574,6 +763,10 @@ class Downwind(Algorithm):
             Flag for ambient instead of waked calculation
         chunked_results: bool
             Flag for chunked results
+        states_sel: list, optional
+            Reduce to selected states
+        states_isel: list, optional
+            Reduce to the selected states indices
         kwargs: dict, optional
             Additional parameters for run_calculation
 
@@ -584,7 +777,6 @@ class Downwind(Algorithm):
             dimensions (state, point)
 
         """
-
         if not self.initialized:
             self.initialize()
         if not ambient and farm_results is None:
@@ -593,7 +785,8 @@ class Downwind(Algorithm):
             )
 
         # welcome:
-        self._print_deco("calc_points", n_points=points.shape[1])
+        points = np.asarray(points)
+        self.print_deco("calc_points", n_points=points.shape[1])
 
         # collect models and initialize:
         mlist, calc_pars = self._collect_point_models(
@@ -602,59 +795,59 @@ class Downwind(Algorithm):
 
         # initialize models:
         if not mlist.initialized:
-            mlist.initialize(self, self.verbosity)
+            mlist.initialize(self, self.verbosity - 1)
+
+        # subset selections:
+        sel = {} if states_sel is None else {FC.STATE: states_sel}
+        isel = {} if states_isel is None else {FC.STATE: states_isel}
+        if states_isel is not None:
+            farm_results = farm_results.isel(isel)
+        if states_sel is not None:
+            farm_results = farm_results.sel(sel)
+        n_states = farm_results.sizes[FC.STATE]
 
         # get input model data:
-        models_data = self.get_models_data()
+        model_data = self.get_models_data(sel=sel, isel=isel)
         if persist_mdata:
-            models_data = models_data.persist()
-        self.print("\nInput data:\n\n", models_data, "\n")
-        self.print(f"\nOutput farm variables:", ", ".join(self.farm_vars))
-        self.print(f"\nChunks: {self.chunks}\n")
+            model_data = model_data.persist()
+        self.print("\nInput data:\n\n", model_data, "\n")
+        self.print("\nOutput farm variables:", ", ".join(self.farm_vars))
 
         # chunk farm results:
-        if self.chunks is not None:
-            farm_results = self.chunked(farm_results)
         self.print("\nInput farm data:\n\n", farm_results, "\n")
 
         # get point data:
         if FC.STATE in farm_results.coords:
             sinds = farm_results.coords[FC.STATE]
-        elif models_data is not None and FC.STATE in models_data.coords:
-            sinds = models_data.coords[FC.STATE]
+        elif model_data is not None and FC.STATE in model_data.coords:
+            sinds = model_data.coords[FC.STATE]
         else:
             sinds = None
-        point_data = self.new_point_data(points, sinds)
+        point_data = self.new_point_data(points, sinds, n_states=n_states)
         if persist_pdata:
             point_data = point_data.persist()
         self.print("\nInput point data:\n\n", point_data, "\n")
 
         # check vars:
-        ovars = mlist.output_point_vars(self)
-        self.print(f"\nOutput point variables:", ", ".join(ovars))
-        self.print(f"\nChunks: {self.chunks}\n")
+        ovars = mlist.output_point_vars(self) if outputs is None else outputs
+        self.print("\nOutput point variables:", ", ".join(ovars))
 
         # calculate:
-        self.print(
-            f"Calculating {len(ovars)} variables at {points.shape[1]} points in {self.n_states} states"
-        )
-
-        point_results = mlist.run_calculation(
-            self,
-            models_data,
+        point_results = super().calc_points(
+            mlist,
+            model_data,
             farm_results,
             point_data,
-            out_vars=ovars,
+            outputs=ovars,
             parameters=calc_pars,
             **kwargs,
         )
-
-        del models_data, farm_results, point_data
+        del model_data, farm_results, point_data
 
         # finalize models:
         if finalize:
             self.print("\n")
-            mlist.finalize(self, self.verbosity)
+            mlist.finalize(self, self.verbosity - 1)
             self.finalize()
 
         if ambient:
@@ -665,18 +858,3 @@ class Downwind(Algorithm):
             point_results = self.chunked(point_results)
 
         return point_results
-
-    def finalize(self, clear_mem=False):
-        """
-        Finalizes the algorithm.
-
-        Parameters
-        ----------
-        clear_mem: bool
-            Clear idata memory
-
-        """
-        for m in self.all_models():
-            m.finalize(self, self.verbosity)
-
-        super().finalize(clear_mem)
