@@ -1,6 +1,6 @@
 from abc import abstractmethod
 
-from foxes.utils import all_subclasses
+from foxes.utils import new_instance
 
 from .farm_data_model import FarmDataModel
 
@@ -60,20 +60,4 @@ class TurbineModel(FarmDataModel):
             Additional parameters for constructor
 
         """
-
-        if tmodel_type is None:
-            return None
-
-        allc = all_subclasses(cls)
-        found = tmodel_type in [scls.__name__ for scls in allc]
-
-        if found:
-            for scls in allc:
-                if scls.__name__ == tmodel_type:
-                    return scls(*args, **kwargs)
-
-        else:
-            estr = "Turbine model type '{}' is not defined, available types are \n {}".format(
-                tmodel_type, sorted([i.__name__ for i in allc])
-            )
-            raise KeyError(estr)
+        return new_instance(cls, tmodel_type, *args, **kwargs)
