@@ -3,7 +3,6 @@ from foxes.utils import DataBook
 from . import farms
 from . import states
 from . import power_ct_curves
-from . import windio
 
 FARM = "farm"
 """ Static wind farm data identifier
@@ -20,11 +19,6 @@ PCTCURVE = "power_ct_curve"
 :group: data
 """
 
-WINDIO = "windio"
-""" Static windio data identifier
-:group: data
-"""
-
 
 class StaticData(DataBook):
     """
@@ -41,4 +35,3 @@ class StaticData(DataBook):
         self.add_data_package(FARM, farms, ".csv")
         self.add_data_package(STATES, states, [".csv", ".csv.gz", ".nc", ".tab"])
         self.add_data_package(PCTCURVE, power_ct_curves, ".csv")
-        self.add_data_package(WINDIO, windio, ".yaml")
