@@ -1,7 +1,7 @@
 import numpy as np
 
 from foxes.core import Turbine
-import foxes.constants as FC
+from foxes.config import config
 
 
 def add_grid(
@@ -12,7 +12,7 @@ def add_grid(
     indices=None,
     names=None,
     verbosity=1,
-    **turbine_parameters
+    **turbine_parameters,
 ):
     """
     Add a regular grid of turbines.
@@ -44,8 +44,8 @@ def add_grid(
     inds = list(np.ndindex(*steps))
     n_turbines = len(inds)
 
-    xy_base = np.array(xy_base, dtype=FC.DTYPE)
-    step_vectors = np.array(step_vectors, dtype=FC.DTYPE)
+    xy_base = np.array(xy_base, dtype=config.dtype_double)
+    step_vectors = np.array(step_vectors, dtype=config.dtype_double)
 
     for i in range(n_turbines):
         xi, yi = inds[i]
@@ -54,7 +54,7 @@ def add_grid(
                 xy=xy_base + xi * step_vectors[0] + yi * step_vectors[1],
                 index=None if indices is None else indices[i],
                 name=None if names is None else names[i],
-                **turbine_parameters
+                **turbine_parameters,
             ),
             verbosity=verbosity,
         )
