@@ -11,7 +11,7 @@ def add_row(
     indices=None,
     names=None,
     verbosity=1,
-    **turbine_parameters
+    **turbine_parameters,
 ):
     """
     Add a single row of turbines.
@@ -47,7 +47,7 @@ def add_row(
                 xy=p0 + i * delta,
                 index=None if indices is None else indices[i],
                 name=None if names is None else names[i],
-                **turbine_parameters
+                **turbine_parameters,
             ),
             verbosity=verbosity,
         )
