@@ -1,6 +1,7 @@
 import pandas as pd
 import numpy as np
 
+from foxes.config import config, get_output_path
 import foxes.variables as FV
 import foxes.constants as FC
 
@@ -41,7 +42,7 @@ def create_random_abl_states(
 
     for v, mm in cols_minmax.items():
         data[v] = np.random.uniform(low=mm[0], high=mm[1], size=(n_states,)).astype(
-            FC.DTYPE
+            config.dtype_double
         )
 
     cmol = var2col.get(FV.MOL, FV.MOL)
@@ -67,7 +68,7 @@ def write_random_abl_states(
     normalize=True,
     verbosity=1,
     digits="auto",
-    **kwargs
+    **kwargs,
 ):
     """
     Writes random abl states to file
@@ -98,8 +99,9 @@ def write_random_abl_states(
 
     """
 
+    fpath = get_output_path(file_path)
     if verbosity:
-        print("Writing file", file_path)
+        print("Writing file", fpath)
 
     data = create_random_abl_states(
         n_states, cols_minmax, var2col, mol_abs_range, normalize
@@ -130,4 +132,4 @@ def write_random_abl_states(
             if d is not None:
                 data[v] = data[v].round(d)
 
-    data.to_csv(file_path, **kwargs)
+    data.to_csv(fpath, **kwargs)
