@@ -1,15 +1,15 @@
 import numpy as np
 import pandas as pd
 from xarray import Dataset, open_dataset
-from pathlib import Path
 from scipy.interpolate import interp1d
 
 from foxes.core import States
 from foxes.utils import PandasFileHelper
 from foxes.data import STATES
+from foxes.config import config, get_input_path
+from foxes.utils import wd2uv, uv2wd
 import foxes.variables as FV
 import foxes.constants as FC
-from foxes.utils import wd2uv, uv2wd
 
 
 class MultiHeightStates(States):
@@ -36,7 +36,7 @@ class MultiHeightStates(States):
     fixed_vars: dict, optional
         Fixed uniform variable values, instead of
         reading from data
-    pd_read_pars: dict, optional
+    read_pars: dict, optional
         pandas file reading parameters
     states_sel: slice or range or list of int
         States subset selection
@@ -58,10 +58,10 @@ class MultiHeightStates(States):
         heights,
         var2col={},
         fixed_vars={},
-        pd_read_pars={},
+        read_pars={},
         states_sel=None,
         states_loc=None,
-        ipars={},
+        **ipars,
     ):
         """
         Constructor.
@@ -79,7 +79,7 @@ class MultiHeightStates(States):
         fixed_vars: dict, optional
             Fixed uniform variable values, instead of
             reading from data
-        pd_read_pars: dict, optional
+        read_pars: dict, optional
             pandas file reading parameters
         states_sel: slice or range or list of int, optional
             States subset selection
@@ -91,20 +91,42 @@ class MultiHeightStates(States):
         """
         super().__init__()
 
-        self.data_source = data_source
-        self.ovars = output_vars
-        self.heights = np.array(heights, dtype=FC.DTYPE)
-        self.rpars = pd_read_pars
+        self.ovars = list(output_vars)
+        self.heights = np.array(heights, dtype=config.dtype_double)
+        self.rpars = read_pars
         self.var2col = var2col
         self.fixed_vars = fixed_vars
         self.ipars = ipars
         self.states_sel = states_sel
         self.states_loc = states_loc
 
+        self._data_source = data_source
         self._solo = None
-        self._weights = None
+        self._inds = None
         self._N = None
 
+    def __repr__(self):
+        berr = self.ipars.get("bounds_error", False)
+        ssel = "" if self.states_sel is None else f"n_states={len(self.states_sel)}, "
+        return f"{type(self).__name__}({ssel}bounds_error={berr})"
+
+    @property
+    def data_source(self):
+        """
+        The data source
+
+        Returns
+        -------
+        s: object
+            The data source
+
+        """
+        if self.running:
+            raise ValueError(
+                f"States '{self.name}': Cannot access data_source while running"
+            )
+        return self._data_source
+
     def reset(self, algo=None, states_sel=None, states_loc=None, verbosity=0):
         """
         Reset the states, optionally select states
@@ -175,23 +197,22 @@ class MultiHeightStates(States):
 
         """
         if not isinstance(self.data_source, pd.DataFrame):
-            if not Path(self.data_source).is_file():
-                if verbosity:
+            self._data_source = get_input_path(self.data_source)
+            if not self.data_source.is_file():
+                if verbosity > 0:
                     print(
                         f"States '{self.name}': Reading static data '{self.data_source}' from context '{STATES}'"
                     )
-                self.data_source = algo.dbook.get_file_path(
-                    STATES, self.data_source, check_raw=False
+                self._data_source = algo.dbook.get_file_path(
+                    STATES, self.data_source.name, check_raw=False
                 )
-                if verbosity:
+                if verbosity > 0:
                     print(f"Path: {self.data_source}")
             elif verbosity:
                 print(f"States '{self.name}': Reading file {self.data_source}")
             rpars = dict(self.RDICT, **self.rpars)
             data = PandasFileHelper().read_file(self.data_source, **rpars)
-            isorg = False
         else:
-            isorg = True
             data = self.data_source
 
         if self.states_sel is not None:
@@ -203,34 +224,31 @@ class MultiHeightStates(States):
         self._inds = data.index.to_numpy()
 
         col_w = self.var2col.get(FV.WEIGHT, FV.WEIGHT)
-        self._weights = np.zeros((self._N, algo.n_turbines), dtype=FC.DTYPE)
+        weights = None
         if col_w in data:
-            self._weights[:] = data[col_w].to_numpy()[:, None]
+            weights = data[col_w].to_numpy()
         elif FV.WEIGHT in self.var2col:
             raise KeyError(
                 f"Weight variable '{col_w}' defined in var2col, but not found in states table columns {data.columns}"
             )
-        else:
-            self._weights[:] = 1.0 / self._N
-            if isorg:
-                data = data.copy()
-            data[col_w] = self._weights[:, 0]
 
         cols = []
         cmap = {}
         self._solo = {}
         for v in self.ovars:
-            vcols = self._find_cols(v, data.columns)
-            if len(vcols) == 1:
-                self._solo[v] = data[vcols[0]].to_numpy()
-            elif len(vcols) > 1:
-                cmap[v] = (len(cols), len(cols) + len(vcols))
-                cols += vcols
+            if v != FV.WEIGHT:
+                vcols = self._find_cols(v, data.columns)
+                if len(vcols) == 1:
+                    self._solo[v] = data[vcols[0]].to_numpy()
+                elif len(vcols) > 1:
+                    cmap[v] = (len(cols), len(cols) + len(vcols))
+                    cols += vcols
         data = data[cols]
 
         self.H = self.var(FV.H)
         self.VARS = self.var("vars")
         self.DATA = self.var("data")
+        self.WEIGHT = self.var(FV.WEIGHT)
 
         idata = super().load_data(algo, verbosity)
 
@@ -244,13 +262,85 @@ class MultiHeightStates(States):
             dims,
             data.to_numpy().reshape(self._N, n_vrs, n_hts),
         )
-
+        if weights is not None:
+            idata["data_vars"][self.WEIGHT] = ((FC.STATE,), weights)
         for v, d in self._solo.items():
             idata["data_vars"][self.var(v)] = ((FC.STATE,), d)
         self._solo = list(self._solo.keys())
 
         return idata
 
+    def set_running(
+        self,
+        algo,
+        data_stash,
+        sel=None,
+        isel=None,
+        verbosity=0,
+    ):
+        """
+        Sets this model status to running, and moves
+        all large data to stash.
+
+        The stashed data will be returned by the
+        unset_running() function after running calculations.
+
+        Parameters
+        ----------
+        algo: foxes.core.Algorithm
+            The calculation algorithm
+        data_stash: dict
+            Large data stash, this function adds data here.
+            Key: model name. Value: dict, large model data
+        sel: dict, optional
+            The subset selection dictionary
+        isel: dict, optional
+            The index subset selection dictionary
+        verbosity: int
+            The verbosity level, 0 = silent
+
+        """
+        super().set_running(algo, data_stash, sel, isel, verbosity)
+
+        data_stash[self.name] = dict(
+            data_source=self._data_source,
+            inds=self._inds,
+        )
+        del self._data_source, self._inds
+
+    def unset_running(
+        self,
+        algo,
+        data_stash,
+        sel=None,
+        isel=None,
+        verbosity=0,
+    ):
+        """
+        Sets this model status to not running, recovering large data
+        from stash
+
+        Parameters
+        ----------
+        algo: foxes.core.Algorithm
+            The calculation algorithm
+        data_stash: dict
+            Large data stash, this function adds data here.
+            Key: model name. Value: dict, large model data
+        sel: dict, optional
+            The subset selection dictionary
+        isel: dict, optional
+            The index subset selection dictionary
+        verbosity: int
+            The verbosity level, 0 = silent
+
+        """
+        super().unset_running(algo, data_stash, sel, isel, verbosity)
+
+        data = data_stash[self.name]
+        self._data_source = data.pop("data_source")
+        self._inds = data.pop("inds")
+
     def size(self):
         """
         The total number of states.
@@ -273,6 +363,8 @@ class MultiHeightStates(States):
             The index labels of states, or None for default integers
 
         """
+        if self.running:
+            raise ValueError(f"States '{self.name}': Cannot access index while running")
         return self._inds
 
     def output_point_vars(self, algo):
@@ -292,25 +384,8 @@ class MultiHeightStates(States):
         """
         return self.ovars
 
-    def weights(self, algo):
-        """
-        The statistical weights of all states.
-
-        Parameters
-        ----------
-        algo: foxes.core.Algorithm
-            The calculation algorithm
-
-        Returns
-        -------
-        weights: numpy.ndarray
-            The weights, shape: (n_states, n_turbines)
-
-        """
-        return self._weights
-
     def calculate(self, algo, mdata, fdata, tdata):
-        """ "
+        """
         The main model calculation.
 
         This function is executed on a single chunk of data,
@@ -335,6 +410,8 @@ class MultiHeightStates(States):
             (n_states, n_targets, n_tpoints)
 
         """
+        self.ensure_output_vars(algo, tdata)
+
         n_states = tdata.n_states
         n_targets = tdata.n_targets
         n_tpoints = tdata.n_tpoints
@@ -344,12 +421,23 @@ class MultiHeightStates(States):
         vrs = list(mdata[self.VARS])
         n_vars = len(vrs)
 
-        coeffs = np.zeros((n_h, n_h), dtype=FC.DTYPE)
+        coeffs = np.zeros((n_h, n_h), dtype=config.dtype_double)
         np.fill_diagonal(coeffs, 1.0)
-        ipars = dict(assume_sorted=True, bounds_error=True)
+        ipars = dict(
+            assume_sorted=True,
+            bounds_error=True,
+            fill_value=(coeffs[0], coeffs[-1]),
+        )
         ipars.update(self.ipars)
         intp = interp1d(h, coeffs, axis=0, **ipars)
-        ires = intp(z)
+        try:
+            ires = intp(z)
+        except ValueError as e:
+            if ipars["bounds_error"]:
+                print(
+                    f"{self.name}: Interpolation height out of bounds. Select 'bounds_error=False' for replacement by lowest or largest height"
+                )
+            raise e
         del coeffs, intp
 
         has_wd = FV.WD in vrs
@@ -386,14 +474,27 @@ class MultiHeightStates(States):
             elif has_wd and v == FV.WS:
                 results[v] = np.linalg.norm(uv, axis=-1)
             elif v in self.fixed_vars:
-                results[v] = np.zeros((n_states, n_targets, n_tpoints), dtype=FC.DTYPE)
+                results[v] = np.zeros(
+                    (n_states, n_targets, n_tpoints), dtype=config.dtype_double
+                )
                 results[v][:] = self.fixed_vars[v]
             elif v in self._solo:
-                results[v] = np.zeros((n_states, n_targets, n_tpoints), dtype=FC.DTYPE)
+                results[v] = np.zeros(
+                    (n_states, n_targets, n_tpoints), dtype=config.dtype_double
+                )
                 results[v][:] = mdata[self.var(v)][:, None, None]
             else:
                 results[v] = ires[vrs.index(v)]
 
+        # add weights:
+        if self.WEIGHT in mdata:
+            tdata[FV.WEIGHT] = mdata[self.WEIGHT][:, None, None]
+        else:
+            tdata[FV.WEIGHT] = np.full(
+                (mdata.n_states, 1, 1), 1 / self._N, dtype=config.dtype_double
+            )
+        tdata.dims[FV.WEIGHT] = (FC.STATE, FC.TARGET, FC.TPOINT)
+
         return results
 
     def finalize(self, algo, verbosity=0):
@@ -410,7 +511,6 @@ class MultiHeightStates(States):
         """
         super().finalize(algo, verbosity)
         self._solo = None
-        self._weights = None
         self._N = None
 
 
@@ -474,18 +574,14 @@ class MultiHeightNCStates(MultiHeightStates):
             data_source,
             *args,
             heights=[],
-            pd_read_pars=None,
+            read_pars=None,
             **kwargs,
         )
         self.state_coord = state_coord
         self.heights = heights
         self.h_coord = h_coord
         self.xr_read_pars = xr_read_pars
-
-        if format_times_func == "default":
-            self.format_times_func = lambda t: t.astype("datetime64[ns]")
-        else:
-            self.format_times_func = format_times_func
+        self._format_times_func = format_times_func
 
     def load_data(self, algo, verbosity=0):
         """
@@ -511,15 +607,16 @@ class MultiHeightNCStates(MultiHeightStates):
 
         """
         if not isinstance(self.data_source, Dataset):
-            if not Path(self.data_source).is_file():
-                if verbosity:
+            self._data_source = get_input_path(self.data_source)
+            if not self.data_source.is_file():
+                if verbosity > 0:
                     print(
                         f"States '{self.name}': Reading static data '{self.data_source}' from context '{STATES}'"
                     )
-                self.data_source = algo.dbook.get_file_path(
-                    STATES, self.data_source, check_raw=False
+                self._data_source = algo.dbook.get_file_path(
+                    STATES, self.data_source.name, check_raw=False
                 )
-                if verbosity:
+                if verbosity > 0:
                     print(f"Path: {self.data_source}")
             elif verbosity:
                 print(f"States '{self.name}': Reading file {self.data_source}")
@@ -534,32 +631,37 @@ class MultiHeightNCStates(MultiHeightStates):
 
         self._N = data.sizes[self.state_coord]
         self._inds = data.coords[self.state_coord].to_numpy()
-        if self.format_times_func is not None:
-            self._inds = self.format_times_func(self._inds)
+
+        if self._format_times_func == "default":
+
+            def format_times_func(t):
+                """little helper function to convert times to datetime64"""
+                return t.astype("datetime64[ns]")
+        else:
+            format_times_func = self._format_times_func
+        if format_times_func is not None:
+            self._inds = format_times_func(self._inds)
 
         w_name = self.var2col.get(FV.WEIGHT, FV.WEIGHT)
-        self._weights = np.zeros((self._N, algo.n_turbines), dtype=FC.DTYPE)
+        weights = None
         if w_name in data.data_vars:
             if data[w_name].dims != (self.state_coord,):
                 raise ValueError(
                     f"Weights data '{w_name}': Expecting dims ({self.state_coord},), got {data[w_name]}"
                 )
-            self._weights[:] = data.data_vars[w_name].to_numpy()[:, None]
+            weights = data.data_vars[w_name].to_numpy()
         elif FV.WEIGHT in self.var2col:
             raise KeyError(
                 f"Weight variable '{w_name}' defined in var2col, but not found in data_vars {list(data.data_vars.keys())}"
             )
-        else:
-            self._weights = np.zeros((self._N, algo.n_turbines), dtype=FC.DTYPE)
-            self._weights[:] = 1.0 / self._N
 
         cols = {}
         self._solo = {}
         for v in self.ovars:
+            if v in self.fixed_vars or v == FV.WEIGHT:
+                continue
             c = self.var2col.get(v, v)
-            if c in self.fixed_vars:
-                pass
-            elif c in data.attrs:
+            if c in data.attrs:
                 self._solo[v] = np.full(self._N, data.attrs)
             elif c in data.data_vars:
                 if data[c].dims == (self.state_coord,):
@@ -583,6 +685,7 @@ class MultiHeightNCStates(MultiHeightStates):
         self.H = self.var(FV.H)
         self.VARS = self.var("vars")
         self.DATA = self.var("data")
+        self.WEIGHT = self.var(FV.WEIGHT)
 
         idata = States.load_data(self, algo, verbosity)
         idata["coords"][self.H] = self.heights
@@ -593,11 +696,15 @@ class MultiHeightNCStates(MultiHeightStates):
             dims,
             np.stack(
                 [data.data_vars[c].to_numpy() for c in cols.values()], axis=1
-            ).astype(FC.DTYPE),
+            ).astype(config.dtype_double),
         )
-
+        if weights is not None:
+            idata["data_vars"][self.WEIGHT] = ((FC.STATE,), weights)
         for v, d in self._solo.items():
-            idata["data_vars"][self.var(v)] = ((FC.STATE,), d.astype(FC.DTYPE))
+            idata["data_vars"][self.var(v)] = (
+                (FC.STATE,),
+                d.astype(config.dtype_double),
+            )
         self._solo = list(self._solo.keys())
 
         return idata
