@@ -6,6 +6,7 @@ from pathlib import Path
 from foxes.core import States, VerticalProfile
 from foxes.utils import PandasFileHelper, read_tab_file
 from foxes.data import STATES
+from foxes.config import config, get_input_path
 import foxes.variables as FV
 import foxes.constants as FC
 
@@ -50,7 +51,7 @@ class StatesTable(States):
         var2col={},
         fixed_vars={},
         profiles={},
-        pd_read_pars={},
+        read_pars={},
         states_sel=None,
         states_loc=None,
     ):
@@ -71,7 +72,7 @@ class StatesTable(States):
         profiles: dict
             Key: output variable name str, Value: str or dict
             or `foxes.core.VerticalProfile`
-        pd_read_pars: dict
+        read_pars: dict
             pandas file reading parameters
         states_sel: slice or range or list of int, optional
             States subset selection
@@ -81,9 +82,8 @@ class StatesTable(States):
         """
         super().__init__()
 
-        self.data_source = data_source
-        self.ovars = output_vars
-        self.rpars = pd_read_pars
+        self.ovars = list(output_vars)
+        self.rpars = read_pars
         self.var2col = var2col
         self.fixed_vars = fixed_vars
         self.profdicts = profiles
@@ -95,10 +95,27 @@ class StatesTable(States):
                 f"States '{self.name}': Cannot handle both 'states_sel' and 'states_loc', please pick one"
             )
 
-        self._weights = None
         self._N = None
         self._tvars = None
         self._profiles = None
+        self._data = data_source
+
+    @property
+    def data_source(self):
+        """
+        The data source
+
+        Returns
+        -------
+        s: object
+            The data source
+
+        """
+        if self.running:
+            raise ValueError(
+                f"States '{self.name}': Cannot access data_source while running"
+            )
+        return self._data
 
     def reset(self, algo=None, states_sel=None, states_loc=None, verbosity=0):
         """
@@ -136,7 +153,6 @@ class StatesTable(States):
 
         """
         self._profiles = {}
-        self._tvars = set(self.ovars)
         for v, d in self.profdicts.items():
             if isinstance(d, str):
                 self._profiles[v] = VerticalProfile.new(d)
@@ -149,9 +165,7 @@ class StatesTable(States):
                 raise TypeError(
                     f"States '{self.name}': Wrong profile type '{type(d).__name__}' for variable '{v}'. Expecting VerticalProfile, str or dict"
                 )
-            self._tvars.update(self._profiles[v].input_vars())
-        self._tvars -= set(self.fixed_vars.keys())
-        self._tvars = list(self._tvars)
+
         super().initialize(algo, verbosity)
 
     def sub_models(self):
@@ -164,7 +178,6 @@ class StatesTable(States):
             Names of all sub models
 
         """
-
         return list(self._profiles.values())
 
     def load_data(self, algo, verbosity=0):
@@ -192,47 +205,48 @@ class StatesTable(States):
         """
         self.VARS = self.var("vars")
         self.DATA = self.var("data")
+        self.WEIGHT = self.var(FV.WEIGHT)
 
         if isinstance(self.data_source, pd.DataFrame):
             data = self.data_source
-            isorg = True
         else:
-            if not Path(self.data_source).is_file():
-                if verbosity:
+            self._data = get_input_path(self.data_source)
+            if not self.data_source.is_file():
+                if verbosity > 0:
                     print(
                         f"States '{self.name}': Reading static data '{self.data_source}' from context '{STATES}'"
                     )
-                self.data_source = algo.dbook.get_file_path(
-                    STATES, self.data_source, check_raw=False
+                self._data = algo.dbook.get_file_path(
+                    STATES, self.data_source.name, check_raw=False
                 )
-                if verbosity:
+                if verbosity > 0:
                     print(f"Path: {self.data_source}")
             elif verbosity:
                 print(f"States '{self.name}': Reading file {self.data_source}")
             rpars = dict(self.RDICT, **self.rpars)
             data = PandasFileHelper().read_file(self.data_source, **rpars)
-            isorg = False
 
         if self.states_sel is not None:
             data = data.iloc[self.states_sel]
         elif self.states_loc is not None:
             data = data.loc[self.states_loc]
         self._N = len(data.index)
-        self._inds = data.index.to_numpy()
+        self.__inds = data.index.to_numpy()
 
         col_w = self.var2col.get(FV.WEIGHT, FV.WEIGHT)
-        self._weights = np.zeros((self._N, algo.n_turbines), dtype=FC.DTYPE)
+        weights = None
         if col_w in data:
-            self._weights[:] = data[col_w].to_numpy()[:, None]
+            weights = data[col_w].to_numpy()
         elif FV.WEIGHT in self.var2col:
             raise KeyError(
                 f"Weight variable '{col_w}' defined in var2col, but not found in states table columns {data.columns}"
             )
-        else:
-            self._weights[:] = 1.0 / self._N
-            if isorg:
-                data = data.copy()
-            data[col_w] = self._weights[:, 0]
+
+        self._tvars = set(self.ovars)
+        for v in self.profdicts.keys():
+            self._tvars.update(self._profiles[v].input_vars())
+        self._tvars -= set(self.fixed_vars.keys())
+        self._tvars = list(self._tvars)
 
         tcols = []
         for v in self._tvars:
@@ -248,6 +262,8 @@ class StatesTable(States):
         idata = super().load_data(algo, verbosity)
         idata["coords"][self.VARS] = self._tvars
         idata["data_vars"][self.DATA] = ((FC.STATE, self.VARS), data.to_numpy())
+        if weights is not None:
+            idata["data_vars"][self.WEIGHT] = (FC.STATE, weights)
 
         return idata
 
@@ -273,7 +289,9 @@ class StatesTable(States):
             The index labels of states, or None for default integers
 
         """
-        return self._inds
+        if self.running:
+            raise ValueError(f"States '{self.name}': Cannot access index while running")
+        return self.__inds
 
     def output_point_vars(self, algo):
         """
@@ -292,25 +310,79 @@ class StatesTable(States):
         """
         return self.ovars
 
-    def weights(self, algo):
+    def set_running(
+        self,
+        algo,
+        data_stash,
+        sel=None,
+        isel=None,
+        verbosity=0,
+    ):
         """
-        The statistical weights of all states.
+        Sets this model status to running, and moves
+        all large data to stash.
+
+        The stashed data will be returned by the
+        unset_running() function after running calculations.
 
         Parameters
         ----------
         algo: foxes.core.Algorithm
             The calculation algorithm
-
-        Returns
-        -------
-        weights: numpy.ndarray
-            The weights, shape: (n_states, n_turbines)
+        data_stash: dict
+            Large data stash, this function adds data here.
+            Key: model name. Value: dict, large model data
+        sel: dict, optional
+            The subset selection dictionary
+        isel: dict, optional
+            The index subset selection dictionary
+        verbosity: int
+            The verbosity level, 0 = silent
 
         """
-        return self._weights
+        super().set_running(algo, data_stash, sel, isel, verbosity)
+
+        data_stash[self.name] = dict(
+            data_source=self._data,
+            inds=self.__inds,
+        )
+        del self._data, self.__inds
+
+    def unset_running(
+        self,
+        algo,
+        data_stash,
+        sel=None,
+        isel=None,
+        verbosity=0,
+    ):
+        """
+        Sets this model status to not running, recovering large data
+        from stash
+
+        Parameters
+        ----------
+        algo: foxes.core.Algorithm
+            The calculation algorithm
+        data_stash: dict
+            Large data stash, this function adds data here.
+            Key: model name. Value: dict, large model data
+        sel: dict, optional
+            The subset selection dictionary
+        isel: dict, optional
+            The index subset selection dictionary
+        verbosity: int
+            The verbosity level, 0 = silent
+
+        """
+        super().unset_running(algo, data_stash, sel, isel, verbosity)
+
+        data = data_stash[self.name]
+        self._data = data.pop("data_source")
+        self.__inds = data.pop("inds")
 
     def calculate(self, algo, mdata, fdata, tdata):
-        """ "
+        """
         The main model calculation.
 
         This function is executed on a single chunk of data,
@@ -335,25 +407,26 @@ class StatesTable(States):
             (n_states, n_targets, n_tpoints)
 
         """
+        self.ensure_output_vars(algo, tdata)
+
         for i, v in enumerate(self._tvars):
-            if v in tdata:
-                tdata[v][:] = mdata[self.DATA][:, i, None, None]
-            else:
-                tdata[v] = np.zeros(
-                    (tdata.n_states, tdata.n_targets, tdata.n_tpoints), dtype=FC.DTYPE
-                )
-                tdata[v][:] = mdata[self.DATA][:, i, None, None]
-                tdata.dims[v] = (FC.STATE, FC.TARGET, FC.TPOINT)
+            tdata[v][:] = mdata[self.DATA][:, i, None, None]
 
         for v, f in self.fixed_vars.items():
-            tdata[v] = np.full(
-                (tdata.n_states, tdata.n_targets, tdata.n_tpoints), f, dtype=FC.DTYPE
-            )
+            tdata[v][:] = f
 
         z = tdata[FC.TARGETS][..., 2]
         for v, p in self._profiles.items():
             tdata[v] = p.calculate(tdata, z)
 
+        if self.WEIGHT in mdata:
+            tdata[FV.WEIGHT] = mdata[self.WEIGHT][:, None, None]
+        else:
+            tdata[FV.WEIGHT] = np.full(
+                (mdata.n_states, 1, 1), 1 / self._N, dtype=config.dtype_double
+            )
+        tdata.dims[FV.WEIGHT] = (FC.STATE, FC.TARGET, FC.TPOINT)
+
         return {v: tdata[v] for v in self.output_point_vars(algo)}
 
     def finalize(self, algo, verbosity=0):
@@ -368,7 +441,6 @@ class StatesTable(States):
             The verbosity level
 
         """
-        self._weights = None
         self._N = None
         self._tvars = None
 
@@ -412,11 +484,11 @@ class TabStates(StatesTable):
         """
         self._normalize = normalize
         if isinstance(data_source, Dataset):
-            self._tab_source = None
-            self._tab_data = data_source
+            self.__tab_source = None
+            self.__tab_data = data_source
         elif isinstance(data_source, (str, Path)):
-            self._tab_source = data_source
-            self._tab_data = None
+            self.__tab_source = data_source
+            self.__tab_data = None
         else:
             raise TypeError(
                 f"Expecting str, Path or xarray.Dataset as data_source, got {type(data_source)}"
@@ -448,47 +520,49 @@ class TabStates(StatesTable):
 
         """
         if self.data_source is None:
-            if self._tab_data is None:
-                if not Path(self._tab_source).is_file():
-                    if verbosity:
+            if self.__tab_data is None:
+                self.__tab_source = get_input_path(self.__tab_source)
+                if not self.__tab_source.is_file():
+                    if verbosity > 0:
                         print(
-                            f"States '{self.name}': Reading static data '{self._tab_source}' from context '{STATES}'"
+                            f"States '{self.name}': Reading static data '{self.__tab_source}' from context '{STATES}'"
                         )
-                    self._tab_source = algo.dbook.get_file_path(
-                        STATES, self._tab_source, check_raw=False
+                    self.__tab_source = algo.dbook.get_file_path(
+                        STATES, self.__tab_source.name, check_raw=False
                     )
-                    if verbosity:
-                        print(f"Path: {self._tab_source}")
+                    if verbosity > 0:
+                        print(f"Path: {self.__tab_source}")
                 elif verbosity:
-                    print(f"States '{self.name}': Reading file {self._tab_source}")
-                self._tab_data = read_tab_file(self._tab_source, self._normalize)
+                    print(f"States '{self.name}': Reading file {self.__tab_source}")
+                self.__tab_data = read_tab_file(self.__tab_source, self._normalize)
 
-            a = self._tab_data.attrs["factor_ws"]
-            b = self._tab_data.attrs["shift_wd"]
+            a = self.__tab_data.attrs["factor_ws"]
+            b = self.__tab_data.attrs["shift_wd"]
             if b != 0.0:
                 raise ValueError(
                     f"{self.name}: shift_wd = {b} is not supported, expecting zero"
                 )
 
-            wd0 = self._tab_data["wd"].to_numpy()
+            wd0 = self.__tab_data["wd"].to_numpy()
             ws0 = a * np.append(
-                np.array([0], dtype=FC.DTYPE), self._tab_data["ws"].to_numpy()
+                np.array([0], dtype=config.dtype_double),
+                self.__tab_data["ws"].to_numpy(),
             )
             ws0 = 0.5 * (ws0[:-1] + ws0[1:])
 
-            n_ws = self._tab_data.sizes["ws"]
-            n_wd = self._tab_data.sizes["wd"]
-            ws = np.zeros((n_ws, n_wd), dtype=FC.DTYPE)
-            wd = np.zeros((n_ws, n_wd), dtype=FC.DTYPE)
+            n_ws = self.__tab_data.sizes["ws"]
+            n_wd = self.__tab_data.sizes["wd"]
+            ws = np.zeros((n_ws, n_wd), dtype=config.dtype_double)
+            wd = np.zeros((n_ws, n_wd), dtype=config.dtype_double)
             ws[:] = ws0[:, None]
             wd[:] = wd0[None, :]
 
-            wd_freq = self._tab_data["wd_freq"].to_numpy() / 100
-            weights = self._tab_data["ws_freq"].to_numpy() * wd_freq[None, :] / 1000
+            wd_freq = self.__tab_data["wd_freq"].to_numpy() / 100
+            weights = self.__tab_data["ws_freq"].to_numpy() * wd_freq[None, :] / 1000
 
             sel = weights > 0
 
-            self.data_source = pd.DataFrame(
+            self._data = pd.DataFrame(
                 index=np.arange(np.sum(sel)),
                 data={
                     FV.WS: ws[sel],
@@ -496,6 +570,79 @@ class TabStates(StatesTable):
                     FV.WEIGHT: weights[sel],
                 },
             )
-            self.data_source.index.name = FC.STATE
+            self._data.index.name = FC.STATE
 
         return super().load_data(algo, verbosity)
+
+    def set_running(
+        self,
+        algo,
+        data_stash,
+        sel=None,
+        isel=None,
+        verbosity=0,
+    ):
+        """
+        Sets this model status to running, and moves
+        all large data to stash.
+
+        The stashed data will be returned by the
+        unset_running() function after running calculations.
+
+        Parameters
+        ----------
+        algo: foxes.core.Algorithm
+            The calculation algorithm
+        data_stash: dict
+            Large data stash, this function adds data here.
+            Key: model name. Value: dict, large model data
+        sel: dict, optional
+            The subset selection dictionary
+        isel: dict, optional
+            The index subset selection dictionary
+        verbosity: int
+            The verbosity level, 0 = silent
+
+        """
+        super().set_running(algo, data_stash, sel, isel, verbosity)
+
+        data_stash[self.name].update(
+            dict(
+                tab_source=self.__tab_source,
+                tab_data=self.__tab_data,
+            )
+        )
+        del self.__tab_source, self.__tab_data
+
+    def unset_running(
+        self,
+        algo,
+        data_stash,
+        sel=None,
+        isel=None,
+        verbosity=0,
+    ):
+        """
+        Sets this model status to not running, recovering large data
+        from stash
+
+        Parameters
+        ----------
+        algo: foxes.core.Algorithm
+            The calculation algorithm
+        data_stash: dict
+            Large data stash, this function adds data here.
+            Key: model name. Value: dict, large model data
+        sel: dict, optional
+            The subset selection dictionary
+        isel: dict, optional
+            The index subset selection dictionary
+        verbosity: int
+            The verbosity level, 0 = silent
+
+        """
+        super().unset_running(algo, data_stash, sel, isel, verbosity)
+
+        data = data_stash[self.name]
+        self.__tab_source = data.pop("tab_source")
+        self.__tab_data = data.pop("tab_data")
