@@ -17,6 +17,7 @@ from foxes.core import (
     AxialInductionModel,
     TurbineInductionModel,
     GroundModel,
+    WakeDeflection,
 )
 
 
@@ -62,6 +63,9 @@ class ModelBook:
     ground_models: foxes.utils.FDict
         The ground models. Keys: model name str,
         values: foxes.core.GroundModel
+    wake_deflections: foxes.utils.FDict
+        The wake deflection models. Keys: model name str,
+        values: foxes.core.WakeDeflection
     sources: foxes.utils.FDict
         All sources dict
     base_classes: foxes.utils.FDict
@@ -81,12 +85,13 @@ class ModelBook:
             Path to power/ct curve file, for creation
             of default turbine type model
         """
-        self.point_models = FDict(name="point_models")
+        self.point_models = FDict(_name="point_models")
         self.point_models["tke2ti"] = fm.point_models.TKE2TI()
+        self.point_models["ustar2ti"] = fm.point_models.Ustar2TI()
 
-        self.rotor_models = FDict(name="rotor_models")
-        rvars = [FV.REWS, FV.REWS2, FV.REWS3, FV.TI, FV.RHO]
-        self.rotor_models["centre"] = fm.rotor_models.CentreRotor(calc_vars=rvars)
+        self.rotor_models = FDict(_name="rotor_models")
+        self.rotor_models["centre"] = fm.rotor_models.CentreRotor()
+        self.rotor_models["direct_mdata"] = fm.rotor_models.DirectMDataInfusion()
 
         def _n2n(n2):
             n2 = float(n2)
@@ -100,7 +105,7 @@ class ModelBook:
         self.rotor_models.add_factory(
             fm.rotor_models.GridRotor,
             "grid<n2>",
-            kwargs=dict(calc_vars=rvars, reduce=True),
+            kwargs=dict(reduce=True),
             var2arg={"n2": "n"},
             n2=_n2n,
             hints={"n2": "(Number of points in square grid)"},
@@ -108,7 +113,7 @@ class ModelBook:
         self.rotor_models.add_factory(
             fm.rotor_models.GridRotor,
             "grid<n2>_raw",
-            kwargs=dict(calc_vars=rvars, reduce=False),
+            kwargs=dict(reduce=False),
             var2arg={"n2": "n"},
             n2=_n2n,
             hints={"n2": "(Number of points in square grid)"},
@@ -116,19 +121,19 @@ class ModelBook:
         self.rotor_models.add_factory(
             fm.rotor_models.LevelRotor,
             "level<n>",
-            kwargs=dict(calc_vars=rvars, reduce=True),
+            kwargs=dict(reduce=True),
             n=lambda x: int(x),
             hints={"n": "(Number of vertical levels)"},
         )
         self.rotor_models.add_factory(
             fm.rotor_models.LevelRotor,
             "level<n>_raw",
-            kwargs=dict(calc_vars=rvars, reduce=False),
+            kwargs=dict(reduce=False),
             n=lambda x: int(x),
             hints={"n": "(Number of vertical levels)"},
         )
 
-        self.turbine_types = FDict(name="turbine_types")
+        self.turbine_types = FDict(_name="turbine_types")
         self.turbine_types["null_type"] = fm.turbine_types.NullType()
         self.turbine_types["NREL5MW"] = fm.turbine_types.PCtFile(
             "NREL-5MW-D126-H90.csv", rho=1.225
@@ -146,7 +151,7 @@ class ModelBook:
             self.turbine_types["Pct"] = fm.turbine_types.PCtFile(Pct_file)
 
         self.turbine_models = FDict(
-            name="turbine_models",
+            _name="turbine_models",
             kTI=fm.turbine_models.kTI(),
             kTI_amb=fm.turbine_models.kTI(ti_var=FV.AMB_TI),
             thrust2ct=fm.turbine_models.Thrust2Ct(),
@@ -158,14 +163,14 @@ class ModelBook:
             fm.turbine_models.kTI,
             "kTI_<kTI>",
             kTI=lambda x: float(f"0.{x[1:]}" if x[0] == "0" else float(x)),
-            hints={"kTI": "(Value, e.g. 004 for 0.04)"},
+            hints={"kTI": "(Value, e.g. 02 for 0.2)"},
         )
         self.turbine_models.add_factory(
             fm.turbine_models.kTI,
             "kTI_amb_<kTI>",
             kwargs=dict(ti_var=FV.AMB_TI),
             kTI=lambda x: float(f"0.{x[1:]}" if x[0] == "0" else float(x)),
-            hints={"kTI": "(Value, e.g. 004 for 0.04)"},
+            hints={"kTI": "(Value, e.g. 04 for 0.4)"},
         )
         self.turbine_models.add_factory(
             fm.turbine_models.kTI,
@@ -173,7 +178,7 @@ class ModelBook:
             kTI=lambda x: float(f"0.{x[1:]}" if x[0] == "0" else float(x)),
             kb=lambda x: float(f"0.{x[1:]}" if x[0] == "0" else float(x)),
             hints={
-                "kTI": "(Value, e.g. 004 for 0.04)",
+                "kTI": "(Value, e.g. 04 for 0.4)",
                 "kb": "(Value, e.g. 004 for 0.04)",
             },
         )
@@ -184,7 +189,7 @@ class ModelBook:
             kTI=lambda x: float(f"0.{x[1:]}" if x[0] == "0" else float(x)),
             kb=lambda x: float(f"0.{x[1:]}" if x[0] == "0" else float(x)),
             hints={
-                "kTI": "(Value, e.g. 004 for 0.04)",
+                "kTI": "(Value, e.g. 04 for 0.4)",
                 "kb": "(Value, e.g. 004 for 0.04)",
             },
         )
@@ -199,7 +204,7 @@ class ModelBook:
         )
 
         self.farm_models = FDict(
-            name="farm_models",
+            _name="farm_models",
             **{
                 f"farm_{mname}": fm.farm_models.Turbine2FarmModel(m)
                 for mname, m in self.turbine_models.items()
@@ -207,12 +212,12 @@ class ModelBook:
         )
 
         self.farm_controllers = FDict(
-            name="farm_controllers",
+            _name="farm_controllers",
             basic_ctrl=fm.farm_controllers.BasicFarmController(),
         )
 
         self.partial_wakes = FDict(
-            name="partial_wakes",
+            _name="partial_wakes",
             rotor_points=fm.partial_wakes.RotorPoints(),
             top_hat=fm.partial_wakes.PartialTopHat(),
             centre=fm.partial_wakes.PartialCentre(),
@@ -231,14 +236,77 @@ class ModelBook:
             hints={"n2": "(Number of points in square grid)"},
         )
 
-        self.wake_frames = FDict(
-            name="wake_frames",
-            rotor_wd=fm.wake_frames.RotorWD(var_wd=FV.WD),
-            rotor_wd_farmo=fm.wake_frames.FarmOrder(),
+        self.wake_deflections = FDict(
+            _name="wake_deflections",
+            no_deflection=fm.wake_deflections.NoDeflection(),
+            Bastankhah2016=fm.wake_deflections.Bastankhah2016Deflection(),
+            Jimenez=fm.wake_deflections.JimenezDeflection(rotate=True),
+            JimenezProj=fm.wake_deflections.JimenezDeflection(rotate=False),
+            JimenezPath=fm.wake_deflections.JimenezDeflection(rotate=None),
         )
-        self.wake_frames.add_k_factory(
-            fm.wake_frames.YawedWakes,
-            "yawed_[wake_k]",
+
+        self.wake_deflections.add_factory(
+            fm.wake_deflections.JimenezDeflection,
+            "Jimenez_b<beta>",
+            beta=lambda x: float(f"0.{x[1:]}" if x[0] == "0" else float(x)),
+            hints={"beta": "(The Jimenez beta coefficient, e.g. 01 for 0.1)"},
+            kwargs=dict(rotate=True),
+        )
+        self.wake_deflections.add_factory(
+            fm.wake_deflections.JimenezDeflection,
+            "Jimenez_b<beta>_dx<dx>",
+            beta=lambda x: float(f"0.{x[1:]}" if x[0] == "0" else float(x)),
+            dx=lambda dx: float(dx),
+            var2arg={"dx": "step_x"},
+            hints={
+                "beta": "(The Jimenez beta coefficient, e.g. 01 for 0.1)",
+                "dx": "(The step size in m for integration along wake path, e.g. 10)",
+            },
+            kwargs=dict(rotate=True),
+        )
+        self.wake_deflections.add_factory(
+            fm.wake_deflections.JimenezDeflection,
+            "JimenezProj_b<beta>",
+            beta=lambda x: float(f"0.{x[1:]}" if x[0] == "0" else float(x)),
+            hints={"beta": "(The Jimenez beta coefficient, e.g. 01 for 0.1)"},
+            kwargs=dict(rotate=False),
+        )
+        self.wake_deflections.add_factory(
+            fm.wake_deflections.JimenezDeflection,
+            "JimenezProj_b<beta>_dx<dx>",
+            beta=lambda x: float(f"0.{x[1:]}" if x[0] == "0" else float(x)),
+            dx=lambda dx: float(dx),
+            var2arg={"dx": "step_x"},
+            hints={
+                "beta": "(The Jimenez beta coefficient, e.g. 01 for 0.1)",
+                "dx": "(The step size in m for integration along wake path, e.g. 10)",
+            },
+            kwargs=dict(rotate=False),
+        )
+        self.wake_deflections.add_factory(
+            fm.wake_deflections.JimenezDeflection,
+            "JimenezPath_b<beta>",
+            beta=lambda x: float(f"0.{x[1:]}" if x[0] == "0" else float(x)),
+            hints={"beta": "(The Jimenez beta coefficient, e.g. 01 for 0.1)"},
+            kwargs=dict(rotate=None),
+        )
+        self.wake_deflections.add_factory(
+            fm.wake_deflections.JimenezDeflection,
+            "JimenezPath_b<beta>_dx<dx>",
+            beta=lambda x: float(f"0.{x[1:]}" if x[0] == "0" else float(x)),
+            dx=lambda dx: float(dx),
+            var2arg={"dx": "step_x"},
+            hints={
+                "beta": "(The Jimenez beta coefficient, e.g. 01 for 0.1)",
+                "dx": "(The step size in m for integration along wake path, e.g. 10)",
+            },
+            kwargs=dict(rotate=None),
+        )
+
+        self.wake_frames = FDict(
+            _name="wake_frames",
+            rotor_wd=fm.wake_frames.RotorWD(),
+            rotor_wd_farmo=fm.wake_frames.FarmOrder(),
         )
         self.wake_frames.add_factory(
             fm.wake_frames.Streamlines2D,
@@ -254,12 +322,24 @@ class ModelBook:
         )
 
         self.wake_frames["timelines"] = fm.wake_frames.Timelines()
+        self.wake_frames["dyn_wakes"] = fm.wake_frames.DynamicWakes()
+        self.wake_frames["seq_dyn_wakes"] = fm.wake_frames.SeqDynamicWakes()
 
         def _todt(x):
             if x[-1] == "s":
                 return float(x[:-1]) / 60
             elif x[-3:] == "min":
                 return float(x[:-3])
+            else:
+                raise NotImplementedError(f"Cannot translate '{x}' into minutes")
+
+        def _tokm(x):
+            if x[-2:] == "km":
+                return float(x[:-2])
+            elif x[-1] == "m":
+                return float(x[:-1]) / 1e3
+            else:
+                raise NotImplementedError(f"Cannot translate '{x}' into km")
 
         self.wake_frames.add_factory(
             fm.wake_frames.Timelines,
@@ -268,6 +348,13 @@ class ModelBook:
             var2arg={"dt": "dt_min"},
             hints={"dt": "(Time step, e.g '10s', '1min' etc.)"},
         )
+        self.wake_frames.add_factory(
+            fm.wake_frames.DynamicWakes,
+            "dyn_wakes_<length>",
+            length=_tokm,
+            var2arg={"length": "max_length_km"},
+            hints={"length": "(Maximal wake length, e.g. '5km' or '5000m')"},
+        )
         self.wake_frames.add_factory(
             fm.wake_frames.SeqDynamicWakes,
             "seq_dyn_wakes_<dt>",
@@ -277,7 +364,7 @@ class ModelBook:
         )
 
         self.wake_superpositions = FDict(
-            name="wake_superpositions",
+            _name="wake_superpositions",
             ws_linear=fm.wake_superpositions.WSLinear(scale_amb=False),
             ws_linear_lim=fm.wake_superpositions.WSLinear(
                 scale_amb=False, lim_low=1e-4
@@ -317,88 +404,128 @@ class ModelBook:
             ti_cubic=fm.wake_superpositions.TIPow(pow=3, superp_to_amb="quadratic"),
             ti_quartic=fm.wake_superpositions.TIPow(pow=4, superp_to_amb="quadratic"),
             ti_max=fm.wake_superpositions.TIMax(superp_to_amb="quadratic"),
+            vector=fm.wake_superpositions.WindVectorLinear(scale_amb=False),
+            vector_amb=fm.wake_superpositions.WindVectorLinear(scale_amb=True),
         )
 
-        self.axial_induction = FDict(name="induction_models")
+        self.axial_induction = FDict(_name="induction_models")
         self.axial_induction["Betz"] = fm.axial_induction.BetzAxialInduction()
         self.axial_induction["Madsen"] = fm.axial_induction.MadsenAxialInduction()
 
-        self.wake_models = FDict(name="wake_models")
+        self.wake_models = FDict(_name="wake_models")
 
         self.wake_models.add_k_factory(
             fm.wake_models.wind.JensenWake,
             "Jensen_<superposition>_[wake_k]",
             kwargs=dict(induction="Betz"),
-            superposition=lambda s: f"ws_{s}",
-            hints={"superposition": "(Superposition, e.g. linear for ws_linear)"},
+            superposition=lambda s: f"ws_{s}"
+            if f"ws_{s}" in self.wake_superpositions
+            else s,
+            hints={
+                "superposition": "(Superposition, e.g. linear for ws_linear, or vector)"
+            },
         )
 
         self.wake_models.add_k_factory(
             fm.wake_models.wind.Bastankhah2014,
             "Bastankhah2014_<superposition>_[wake_k]",
             kwargs=dict(sbeta_factor=0.2, induction="Madsen"),
-            superposition=lambda s: f"ws_{s}",
-            hints={"superposition": "(Superposition, e.g. linear for ws_linear)"},
+            superposition=lambda s: f"ws_{s}"
+            if f"ws_{s}" in self.wake_superpositions
+            else s,
+            hints={
+                "superposition": "(Superposition, e.g. linear for ws_linear, or vector)"
+            },
         )
         self.wake_models.add_k_factory(
             fm.wake_models.wind.Bastankhah2014,
             "Bastankhah2014B_<superposition>_[wake_k]",
             kwargs=dict(sbeta_factor=0.2, induction="Betz"),
-            superposition=lambda s: f"ws_{s}",
-            hints={"superposition": "(Superposition, e.g. linear for ws_linear)"},
+            superposition=lambda s: f"ws_{s}"
+            if f"ws_{s}" in self.wake_superpositions
+            else s,
+            hints={
+                "superposition": "(Superposition, e.g. linear for ws_linear, or vector)"
+            },
         )
         self.wake_models.add_k_factory(
             fm.wake_models.wind.Bastankhah2014,
             "Bastankhah025_<superposition>_[wake_k]",
             kwargs=dict(sbeta_factor=0.25, induction="Madsen"),
-            superposition=lambda s: f"ws_{s}",
-            hints={"superposition": "(Superposition, e.g. linear for ws_linear)"},
+            superposition=lambda s: f"ws_{s}"
+            if f"ws_{s}" in self.wake_superpositions
+            else s,
+            hints={
+                "superposition": "(Superposition, e.g. linear for ws_linear, or vector)"
+            },
         )
         self.wake_models.add_k_factory(
             fm.wake_models.wind.Bastankhah2014,
             "Bastankhah025B_<superposition>_[wake_k]",
             kwargs=dict(sbeta_factor=0.25, induction="Betz"),
-            superposition=lambda s: f"ws_{s}",
-            hints={"superposition": "(Superposition, e.g. linear for ws_linear)"},
+            superposition=lambda s: f"ws_{s}"
+            if f"ws_{s}" in self.wake_superpositions
+            else s,
+            hints={
+                "superposition": "(Superposition, e.g. linear for ws_linear, or vector)"
+            },
         )
 
         self.wake_models.add_k_factory(
             fm.wake_models.wind.Bastankhah2016,
             "Bastankhah2016_<superposition>_[wake_k]",
             kwargs=dict(induction="Madsen"),
-            superposition=lambda s: f"ws_{s}",
-            hints={"superposition": "(Superposition, e.g. linear for ws_linear)"},
+            superposition=lambda s: f"ws_{s}"
+            if f"ws_{s}" in self.wake_superpositions
+            else s,
+            hints={
+                "superposition": "(Superposition, e.g. linear for ws_linear, or vector)"
+            },
         )
         self.wake_models.add_k_factory(
             fm.wake_models.wind.Bastankhah2016,
             "Bastankhah2016B_<superposition>_[wake_k]",
             kwargs=dict(induction="Betz"),
-            superposition=lambda s: f"ws_{s}",
-            hints={"superposition": "(Superposition, e.g. linear for ws_linear)"},
+            superposition=lambda s: f"ws_{s}"
+            if f"ws_{s}" in self.wake_superpositions
+            else s,
+            hints={
+                "superposition": "(Superposition, e.g. linear for ws_linear, or vector)"
+            },
         )
 
         self.wake_models.add_k_factory(
             fm.wake_models.wind.TurbOParkWake,
             "TurbOPark_<superposition>_[wake_k]",
             kwargs=dict(induction="Madsen"),
-            superposition=lambda s: f"ws_{s}",
-            hints={"superposition": "(Superposition, e.g. linear for ws_linear)"},
+            superposition=lambda s: f"ws_{s}"
+            if f"ws_{s}" in self.wake_superpositions
+            else s,
+            hints={
+                "superposition": "(Superposition, e.g. linear for ws_linear, or vector)"
+            },
         )
         self.wake_models.add_k_factory(
             fm.wake_models.wind.TurbOParkWake,
             "TurbOParkB_<superposition>_[wake_k]",
             kwargs=dict(induction="Betz"),
-            superposition=lambda s: f"ws_{s}",
-            hints={"superposition": "(Superposition, e.g. linear for ws_linear)"},
+            superposition=lambda s: f"ws_{s}"
+            if f"ws_{s}" in self.wake_superpositions
+            else s,
+            hints={
+                "superposition": "(Superposition, e.g. linear for ws_linear, or vector)"
+            },
         )
 
         self.wake_models.add_k_factory(
             fm.wake_models.wind.TurbOParkWakeIX,
             "TurbOParkIX_<superposition>_[wake_k]_dx<dx>",
-            superposition=lambda s: f"ws_{s}",
+            superposition=lambda s: f"ws_{s}"
+            if f"ws_{s}" in self.wake_superpositions
+            else s,
             dx=lambda x: float(x),
             hints={
-                "superposition": "(Superposition, e.g. linear for ws_linear)",
+                "superposition": "(Superposition, e.g. linear for ws_linear, or vector)",
                 "dx": "(Integration step in m)",
             },
         )
@@ -411,14 +538,6 @@ class ModelBook:
             hints={"superposition": "(Superposition, e.g. linear for ti_linear)"},
         )
 
-        self.wake_models.add_factory(
-            fm.wake_models.ti.IECTIWake,
-            "IECTI2005_<superposition>",
-            kwargs=dict(iec_type="2005"),
-            superposition=lambda s: f"ti_{s}",
-            hints={"superposition": "(Superposition, e.g. linear for ti_linear)"},
-        )
-
         self.wake_models.add_factory(
             fm.wake_models.ti.IECTIWake,
             "IECTI2019_<superposition>",
@@ -449,15 +568,56 @@ class ModelBook:
             hints={"superposition": "(Superposition, e.g. linear for ti_linear)"},
         )
 
-        self.wake_models[f"RHB"] = fm.wake_models.induction.RankineHalfBody()
-        self.wake_models[f"Rathmann"] = fm.wake_models.induction.Rathmann()
-        self.wake_models[f"SelfSimilar"] = fm.wake_models.induction.SelfSimilar()
-        self.wake_models[f"SelfSimilar2020"] = (
-            fm.wake_models.induction.SelfSimilar2020()
-        )
-        self.wake_models[f"VortexSheet"] = fm.wake_models.induction.VortexSheet()
+        self.wake_models["RHB"] = fm.wake_models.induction.RankineHalfBody()
 
-        self.ground_models = FDict(name="ground_models")
+        self.wake_models["VortexSheet"] = fm.wake_models.induction.VortexSheet()
+        self.wake_models.add_factory(
+            fm.wake_models.induction.VortexSheet,
+            "VortexSheet_<superposition>",
+            superposition=lambda s: f"ws_{s}"
+            if f"ws_{s}" in self.wake_superpositions
+            else s,
+            hints={
+                "superposition": "(Superposition, e.g. linear for ws_linear, or vector)"
+            },
+        )
+
+        self.wake_models["Rathmann"] = fm.wake_models.induction.Rathmann()
+        self.wake_models.add_factory(
+            fm.wake_models.induction.Rathmann,
+            "Rathmann_<superposition>",
+            superposition=lambda s: f"ws_{s}"
+            if f"ws_{s}" in self.wake_superpositions
+            else s,
+            hints={
+                "superposition": "(Superposition, e.g. linear for ws_linear, or vector)"
+            },
+        )
+
+        self.wake_models["SelfSimilar"] = fm.wake_models.induction.SelfSimilar()
+        self.wake_models["SelfSimilar2020"] = fm.wake_models.induction.SelfSimilar2020()
+        self.wake_models.add_factory(
+            fm.wake_models.induction.SelfSimilar,
+            "SelfSimilar_<superposition>",
+            superposition=lambda s: f"ws_{s}"
+            if f"ws_{s}" in self.wake_superpositions
+            else s,
+            hints={
+                "superposition": "(Superposition, e.g. linear for ws_linear, or vector)"
+            },
+        )
+        self.wake_models.add_factory(
+            fm.wake_models.induction.SelfSimilar2020,
+            "SelfSimilar2020_<superposition>",
+            superposition=lambda s: f"ws_{s}"
+            if f"ws_{s}" in self.wake_superpositions
+            else s,
+            hints={
+                "superposition": "(Superposition, e.g. linear for ws_linear, or vector)"
+            },
+        )
+
+        self.ground_models = FDict(_name="ground_models")
         self.ground_models["no_ground"] = fm.ground_models.NoGround()
         self.ground_models["ground_mirror"] = fm.ground_models.GroundMirror()
         self.ground_models.add_factory(
@@ -466,10 +626,11 @@ class ModelBook:
             var2arg={"height": "heights"},
             height=lambda h: [0.0, float(h)],
             hints={"height": "(Boundary layer wake reflection height)"},
+            example_vars={"height": 500},
         )
 
         self.sources = FDict(
-            name="sources",
+            _name="sources",
             point_models=self.point_models,
             rotor_models=self.rotor_models,
             turbine_types=self.turbine_types,
@@ -478,13 +639,14 @@ class ModelBook:
             farm_controllers=self.farm_controllers,
             partial_wakes=self.partial_wakes,
             wake_frames=self.wake_frames,
+            wake_deflections=self.wake_deflections,
             wake_superpositions=self.wake_superpositions,
             wake_models=self.wake_models,
             axial_induction=self.axial_induction,
             ground_models=self.ground_models,
         )
         self.base_classes = FDict(
-            name="base_classes",
+            _name="base_classes",
             point_models=PointDataModel,
             rotor_models=RotorModel,
             turbine_types=TurbineType,
@@ -493,6 +655,7 @@ class ModelBook:
             farm_controllers=FarmController,
             partial_wakes=PartialWakesModel,
             wake_frames=WakeFrame,
+            wake_deflections=WakeDeflection,
             wake_superpositions=WakeSuperposition,
             wake_models=WakeModel,
             axial_induction=AxialInductionModel,
@@ -504,7 +667,7 @@ class ModelBook:
                 m.name = k
 
     def __getitem__(self, key):
-        return self.sources.__getitem__(key)
+        return self.sources.get_item(key)
 
     def print_toc(self, subset=None, search=None):
         """
@@ -524,16 +687,19 @@ class ModelBook:
             if subset is None or k in subset:
                 print(k)
                 print("-" * len(k))
-                if len(ms):
-                    for mname in sorted(list(ms.keys())):
-                        if search is None or search in mname:
-                            print(f"{mname}: {ms[mname]}")
-                    if isinstance(ms, FDict):
-                        for f in ms.factories:
-                            if search is None or search in f.name_template:
+                any = False
+                for mname in sorted(list(ms.keys())):
+                    if search is None or search in mname:
+                        print(f"{mname}: {ms[mname]}")
+                        any = True
+                if isinstance(ms, FDict):
+                    for f in ms.factories:
+                        if search is None or search in f.name_template:
+                            if any:
                                 print()
-                                print(f)
-                else:
+                            print(f)
+                            any = True
+                if not any:
                     print("(none)")
                 print()
 
