@@ -3,7 +3,7 @@ import numpy as np
 from foxes.core import RotorModel
 from foxes.utils import wd2uv, uv2wd
 import foxes.variables as FV
-import foxes.constants as FC
+from foxes.config import config
 
 
 class CentreRotor(RotorModel):
@@ -89,7 +89,7 @@ class CentreRotor(RotorModel):
         mdata,
         fdata,
         rpoint_results,
-        weights,
+        rpoint_weights,
         downwind_index=None,
         copy_to_ambient=False,
     ):
@@ -124,14 +124,18 @@ class CentreRotor(RotorModel):
             variables after calculation
 
         """
-        if len(weights) > 1:
+        if len(rpoint_weights) > 1:
             return super().eval_rpoint_results(
-                algo, mdata, fdata, rpoint_results, weights, downwind_index
+                algo, mdata, fdata, rpoint_results, rpoint_weights, downwind_index
             )
 
         n_states = mdata.n_states
         n_turbines = algo.n_turbines
 
+        for v in [FV.REWS2, FV.REWS3]:
+            if v in fdata and v not in self.calc_vars:
+                self.calc_vars.append(v)
+
         uvp = None
         uv = None
         if (
@@ -151,7 +155,7 @@ class CentreRotor(RotorModel):
         vdone = []
         for v in self.calc_vars:
             if v not in fdata:
-                fdata[v] = np.zeros((n_states, n_turbines), dtype=FC.DTYPE)
+                fdata[v] = np.zeros((n_states, n_turbines), dtype=config.dtype_double)
 
             if v == FV.WD or v == FV.YAW:
                 if wd is None:
@@ -188,9 +192,11 @@ class CentreRotor(RotorModel):
         del uvp
 
         for v in self.calc_vars:
-            if v not in vdone:
+            if v not in vdone and (
+                fdata[v].shape[1] > 1 or downwind_index is None or downwind_index == 0
+            ):
                 res = rpoint_results[v][:, :, 0]
                 self._set_res(fdata, v, res, downwind_index)
                 del res
             if copy_to_ambient and v in FV.var2amb:
-                fdata[FV.var2amb[v]] = fdata[v].copy()
+                fdata.add(FV.var2amb[v], fdata[v].copy(), fdata.dims[v])
