@@ -4,7 +4,7 @@ from scipy.interpolate import interpn
 
 from foxes.core import TurbineModel
 from foxes.utils import PandasFileHelper
-import foxes.constants as FC
+from foxes.config import config, get_input_path
 
 
 class TableFactors(TurbineModel):
@@ -106,15 +106,16 @@ class TableFactors(TurbineModel):
         if isinstance(self.data_source, pd.DataFrame):
             self._data = self.data_source
         else:
+            fpath = get_input_path(self.data_source)
             if verbosity > 0:
-                print(f"{self.name}: Reading file {self.data_source}")
+                print(f"{self.name}: Reading file {fpath}")
             rpars = dict(index_col=0)
             rpars.update(self._rpars)
-            self._data = PandasFileHelper.read_file(self.data_source, **rpars)
+            self._data = PandasFileHelper.read_file(fpath, **rpars)
 
-        self._rvals = self._data.index.to_numpy(FC.DTYPE)
-        self._cvals = self._data.columns.to_numpy(FC.DTYPE)
-        self._data = self._data.to_numpy(FC.DTYPE)
+        self._rvals = self._data.index.to_numpy(config.dtype_double)
+        self._cvals = self._data.columns.to_numpy(config.dtype_double)
+        self._data = self._data.to_numpy(config.dtype_double)
 
     def calculate(self, algo, mdata, fdata, st_sel):
         """
@@ -142,8 +143,10 @@ class TableFactors(TurbineModel):
             Values: numpy.ndarray with shape (n_states, n_turbines)
 
         """
+        self.ensure_output_vars(algo, fdata)
+
         n_sel = np.sum(st_sel)
-        qts = np.zeros((n_sel, 2), dtype=FC.DTYPE)
+        qts = np.zeros((n_sel, 2), dtype=config.dtype_double)
         qts[:, 0] = fdata[self.row_var][st_sel]
         qts[:, 1] = fdata[self.col_var][st_sel]
 
