@@ -1,6 +1,5 @@
 from foxes.core import TurbineModel
 import foxes.variables as FV
-import foxes.constants as FC
 from foxes.utils import delta_wd
 
 
@@ -56,6 +55,8 @@ class YAW2YAWM(TurbineModel):
             Values: numpy.ndarray with shape (n_states, n_turbines)
 
         """
+        self.ensure_output_vars(algo, fdata)
+
         yaw = fdata[FV.YAW][st_sel]
         wd = fdata[FV.WD][st_sel]
 
