@@ -1,9 +1,10 @@
 import numpy as np
 
-import foxes.constants as FC
-from .rotor_wd import RotorWD
+from foxes.config import config
 from foxes.core import WakeFrame
 
+from .rotor_wd import RotorWD
+
 
 class FarmOrder(WakeFrame):
     """
@@ -23,7 +24,7 @@ class FarmOrder(WakeFrame):
 
     """
 
-    def __init__(self, base_frame=RotorWD()):
+    def __init__(self, base_frame=None, **kwargs):
         """
         Constructor.
 
@@ -31,11 +32,31 @@ class FarmOrder(WakeFrame):
         ----------
         base_frame: foxes.core.WakeFrame
             The wake frame from which to start
+        kwargs: dict, optional
+            Additional parameters for the base class
 
         """
-        super().__init__()
+        super().__init__(**kwargs)
         self.base_frame = base_frame
 
+    def initialize(self, algo, verbosity=0, force=False):
+        """
+        Initializes the model.
+
+        Parameters
+        ----------
+        algo: foxes.core.Algorithm
+            The calculation algorithm
+        verbosity: int
+            The verbosity level, 0 = silent
+        force: bool
+            Overwrite existing data
+
+        """
+        if self.base_frame is None:
+            self.base_frame = RotorWD()
+        super().initialize(algo, verbosity, force)
+
     def sub_models(self):
         """
         List of all sub-models
@@ -43,13 +64,13 @@ class FarmOrder(WakeFrame):
         Returns
         -------
         smdls: list of foxes.core.Model
-            Names of all sub models
+            All sub models
 
         """
         return [self.base_frame]
 
     def calc_order(self, algo, mdata, fdata):
-        """ "
+        """
         Calculates the order of turbine evaluation.
 
         This function is executed on a single chunk of data,
@@ -70,7 +91,7 @@ class FarmOrder(WakeFrame):
             The turbine order, shape: (n_states, n_turbines)
 
         """
-        order = np.zeros((fdata.n_states, fdata.n_turbines), dtype=FC.ITYPE)
+        order = np.zeros((fdata.n_states, fdata.n_turbines), dtype=config.dtype_int)
         order[:] = np.arange(fdata.n_turbines)[None, :]
 
         return order
@@ -98,7 +119,7 @@ class FarmOrder(WakeFrame):
             The target point data
         downwind_index: int
             The index of the wake causing turbine
-            in the downwnd order
+            in the downwind order
 
         Returns
         -------
