@@ -2,6 +2,7 @@ import numpy as np
 
 from foxes.models.wake_models.dist_sliced import DistSlicedWakeModel
 from foxes.core import Model, WakeK
+from foxes.config import config
 import foxes.variables as FV
 import foxes.constants as FC
 
@@ -146,13 +147,13 @@ class Bastankhah2016Model(Model):
             The k parameter values, shape: (n_states, n_targets)
 
         """
-
         # store parameters:
         out = {self.PARS: self.pars}
         out[self.CHECK] = (
-            mdata[FC.STATE][0],
+            mdata.states_i0(counter=True),
+            mdata.n_states,
             downwind_index,
-            x.shape,
+            hash(x.tobytes()),
         )
 
         # get D:
@@ -180,7 +181,7 @@ class Bastankhah2016Model(Model):
         )
 
         # select targets:
-        st_sel = (x > 0) & (ct > 0.0)
+        st_sel = (x > 1e-8) & (ct > 1e-8)
         if np.any(st_sel):
             # get ws:
             ws = super().get_data(
@@ -191,7 +192,8 @@ class Bastankhah2016Model(Model):
                 fdata=fdata,
                 tdata=tdata,
                 downwind_index=downwind_index,
-                upcast=True,
+                upcast=False,
+                selection=st_sel,
             )
 
             # get TI:
@@ -203,7 +205,8 @@ class Bastankhah2016Model(Model):
                 fdata=fdata,
                 tdata=tdata,
                 downwind_index=downwind_index,
-                upcast=True,
+                upcast=False,
+                selection=st_sel,
             )
 
             # get alpha:
@@ -215,7 +218,8 @@ class Bastankhah2016Model(Model):
                 fdata=fdata,
                 tdata=tdata,
                 downwind_index=downwind_index,
-                upcast=True,
+                upcast=False,
+                selection=st_sel,
             )
 
             # get beta:
@@ -227,19 +231,16 @@ class Bastankhah2016Model(Model):
                 fdata=fdata,
                 tdata=tdata,
                 downwind_index=downwind_index,
-                upcast=True,
+                upcast=False,
+                selection=st_sel,
             )
 
             # apply filter:
             x = x[st_sel]
             D = D[st_sel]
             ct = ct[st_sel]
-            ws = ws[st_sel]
-            ti = ti[st_sel]
             k = k[st_sel]
             gamma = gamma[st_sel]
-            alpha = alpha[st_sel]
-            beta = beta[st_sel]
 
             # calc theta_c0, Eq. (6.12):
             cosg = np.cos(gamma)
@@ -329,7 +330,7 @@ class Bastankhah2016Model(Model):
 
         # update mdata:
         out[self.ST_SEL] = st_sel
-        mdata[self.MDATA_KEY] = out
+        mdata.add(self.MDATA_KEY, out, None)
 
     def has_data(self, mdata, downwind_index, x):
         """
@@ -352,9 +353,10 @@ class Bastankhah2016Model(Model):
 
         """
         check = (
-            mdata[FC.STATE][0],
+            mdata.states_i0(counter=True),
+            mdata.n_states,
             downwind_index,
-            x.shape,
+            hash(x.tobytes()),
         )
         return self.MDATA_KEY in mdata and mdata[self.MDATA_KEY][self.CHECK] == check
 
@@ -445,7 +447,7 @@ class Bastankhah2016(DistSlicedWakeModel):
             Parameters for the WakeK class
 
         """
-        super().__init__(superpositions={FV.WS: superposition})
+        super().__init__(wind_superposition=superposition)
 
         self.model = None
         self.alpha = alpha
@@ -458,10 +460,23 @@ class Bastankhah2016(DistSlicedWakeModel):
     def __repr__(self):
         iname = self.induction
         s = f"{type(self).__name__}"
-        s += f"({self.superpositions[FV.WS]}, induction={iname}, "
+        s += f"({self.wind_superposition}, induction={iname}, "
         s += self.wake_k.repr() + ")"
         return s
 
+    @property
+    def affects_ws(self):
+        """
+        Flag for wind speed wake models
+
+        Returns
+        -------
+        dws: bool
+            If True, this model affects wind speed
+
+        """
+        return True
+
     def sub_models(self):
         """
         List of all sub-models
@@ -472,7 +487,7 @@ class Bastankhah2016(DistSlicedWakeModel):
             Names of all sub models
 
         """
-        return [self.wake_k, self.model]
+        return super().sub_models() + [self.wake_k, self.model]
 
     def initialize(self, algo, verbosity=0, force=False):
         """
@@ -551,7 +566,7 @@ class Bastankhah2016(DistSlicedWakeModel):
                 upcast=True,
                 downwind_index=downwind_index,
             )
-            gamma *= np.pi / 180
+            gamma = gamma * np.pi / 180
 
             # get k:
             k = self.wake_k(
@@ -569,7 +584,7 @@ class Bastankhah2016(DistSlicedWakeModel):
         # select targets:
         st_sel = self.model.get_data(Bastankhah2016Model.ST_SEL, mdata)
         n_sp_sel = np.sum(st_sel)
-        wdeltas = {FV.WS: np.zeros((n_sp_sel, n_y_per_z), dtype=FC.DTYPE)}
+        wdeltas = {FV.WS: np.zeros((n_sp_sel, n_y_per_z), dtype=config.dtype_double)}
         if np.any(st_sel):
             # apply filter:
             yz = yz[st_sel]
@@ -619,4 +634,23 @@ class Bastankhah2016(DistSlicedWakeModel):
                     * np.exp(-0.5 * (z / sigma_z) ** 2)
                 )
 
+        # wake deflection causes wind vector rotation:
+        if FC.WDEFL_ROT_ANGLE in tdata:
+            dwd_defl = tdata.pop(FC.WDEFL_ROT_ANGLE)
+            if FV.WD not in wdeltas:
+                wdeltas[FV.WD] = np.zeros_like(wdeltas[FV.WS])
+                wdeltas[FV.WD][:] = dwd_defl[st_sel]
+            else:
+                wdeltas[FV.WD] += dwd_defl[st_sel]
+
+        # wake deflection causes wind speed reduction:
+        if FC.WDEFL_DWS_FACTOR in tdata:
+            dws_defl = tdata.pop(FC.WDEFL_DWS_FACTOR)
+            if FV.WS not in wdeltas:
+                raise AssertionError(
+                    f"Wake model '{self.name}': Expecting '{FV.WS}' in wdeltas, found {list(wdeltas.keys())}"
+                )
+            else:
+                wdeltas[FV.WS] *= dws_defl[st_sel]
+
         return wdeltas, st_sel
