@@ -4,6 +4,7 @@ from cycler import cycler
 import matplotlib.pyplot as plt
 
 from .output import Output
+from foxes.config import config
 import foxes.variables as FV
 import foxes.constants as FC
 
@@ -24,7 +25,7 @@ class FarmResultsEval(Output):
 
     """
 
-    def __init__(self, farm_results):
+    def __init__(self, farm_results, **kwargs):
         """
         Constructor.
 
@@ -32,8 +33,11 @@ class FarmResultsEval(Output):
         ----------
         farm_results: xarray.Dataset
             The farm results
+        kwargs: dict, optional
+            Additional parameters for the base class
 
         """
+        super().__init__(**kwargs)
         self.results = farm_results
 
     def weinsum(self, rhs, *vars):
@@ -63,29 +67,58 @@ class FarmResultsEval(Output):
         fields = []
         for v in vars:
             if isinstance(v, str):
-                fields.append(self.results[v].to_numpy())
+                vdata = self.results[v].to_numpy()
+                nns = np.sum(np.isnan(vdata))
+                assert nns == 0, (
+                    f"Found {nns} nan values for variable '{v}' of shape {vdata.shape}"
+                )
+                fields.append(vdata)
             else:
                 fields.append(v)
             if nas is None:
                 nas = np.zeros_like(fields[-1], dtype=bool)
             nas = nas | np.isnan(fields[-1])
 
-        inds = ["st" for v in fields] + ["st"]
-        expr = ",".join(inds) + "->" + rhs
+        inds = ["st" for __ in fields]
+        if self.results[FV.WEIGHT].dims == (FC.STATE,):
+            inds += ["s"]
 
-        if np.any(nas):
-            sel = ~np.any(nas, axis=1)
-            fields = [f[sel] for f in fields]
+            if np.any(nas):
+                sel = ~np.any(nas, axis=1)
+                fields = [f[sel] for f in fields]
 
-            weights0 = self.results[FV.WEIGHT].to_numpy()
-            w0 = np.sum(weights0, axis=0)[None, :]
-            weights = weights0[sel]
-            w1 = np.sum(weights, axis=0)[None, :]
-            weights *= w0 / w1
-            fields.append(weights)
+                weights0 = self.results[FV.WEIGHT].to_numpy()
+                w0 = np.sum(weights0)
+                weights = weights0[sel]
+                w1 = np.sum(weights)
+                weights *= w0 / w1
+                fields.append(weights)
+
+            else:
+                fields.append(self.results[FV.WEIGHT].to_numpy())
+
+        elif self.results[FV.WEIGHT].dims == (FC.STATE, FC.TURBINE):
+            inds += ["st"]
+
+            if np.any(nas):
+                sel = ~np.any(nas, axis=1)
+                fields = [f[sel] for f in fields]
+
+                weights0 = self.results[FV.WEIGHT].to_numpy()
+                w0 = np.sum(weights0, axis=0)[None, :]
+                weights = weights0[sel]
+                w1 = np.sum(weights, axis=0)[None, :]
+                weights *= w0 / w1
+                fields.append(weights)
+
+            else:
+                fields.append(self.results[FV.WEIGHT].to_numpy())
 
         else:
-            fields.append(self.results[FV.WEIGHT].to_numpy())
+            raise ValueError(
+                f"Expecting '{FV.WEIGHT}' variable with dimensions {(FC.STATE,)} or {(FC.STATE, FC.TURBINE)}, got {self.results[FV.WEIGHT].dims}"
+            )
+        expr = ",".join(inds) + "->" + rhs
 
         return np.einsum(expr, *fields)
 
@@ -98,7 +131,7 @@ class FarmResultsEval(Output):
         vars_op: dict
             The operation per variable. Key: str, the variable
             name. Value: str, the operation, choices
-            are: sum, mean, min, max.
+            are: weights, mean_no_weights, sum, min, max.
 
         Returns
         -------
@@ -110,23 +143,27 @@ class FarmResultsEval(Output):
 
         rdata = {}
         for v, op in vars_op.items():
-            if op == "mean":
-                rdata[v] = self.weinsum("t", v)
+            vdata = self.results[v].to_numpy()
+            nns = np.sum(np.isnan(vdata))
+            assert nns == 0, (
+                f"Found {nns} nan values for variable '{v}' of shape {vdata.shape}"
+            )
+
+            if op == "weights":
+                rdata[v] = self.weinsum("t", vdata)
+            elif op == "mean_no_weights":
+                rdata[v] = np.mean(vdata, axis=0)
             elif op == "sum":
-                vdata = self.results[v].to_numpy()
                 rdata[v] = np.sum(vdata, axis=0)
             elif op == "min":
-                vdata = self.results[v].to_numpy()
                 rdata[v] = np.min(vdata, axis=0)
             elif op == "max":
-                vdata = self.results[v].to_numpy()
                 rdata[v] = np.max(vdata, axis=0)
             elif op == "std":
-                vdata = self.results[v].to_numpy()
                 rdata[v] = np.std(vdata, axis=0)
             else:
                 raise KeyError(
-                    f"Unknown operation '{op}' for variable '{v}'. Please choose: sum, mean, min, max"
+                    f"Unknown operation '{op}' for variable '{v}'. Please choose: weights, mean_no_weights, sum, min, max"
                 )
 
         data = pd.DataFrame(index=range(n_turbines), data=rdata)
@@ -143,7 +180,7 @@ class FarmResultsEval(Output):
         vars_op: dict
             The operation per variable. Key: str, the variable
             name. Value: str, the operation, choices
-            are: sum, mean, min, max.
+            are: weights, mean_no_weights, sum, min, max.
 
         Returns
         -------
@@ -155,20 +192,25 @@ class FarmResultsEval(Output):
 
         rdata = {}
         for v, op in vars_op.items():
-            if op == "mean":
-                rdata[v] = self.weinsum("s", v)
+            vdata = self.results[v].to_numpy()
+            nns = np.sum(np.isnan(vdata))
+            assert nns == 0, (
+                f"Found {nns} nan values for variable '{v}' of shape {vdata.shape}"
+            )
+
+            if op == "weights":
+                rdata[v] = self.weinsum("s", vdata)
+            elif op == "mean_no_weights":
+                rdata[v] = np.mean(vdata, axis=1)
             elif op == "sum":
-                vdata = self.results[v].to_numpy()
                 rdata[v] = np.sum(vdata, axis=1)
             elif op == "min":
-                vdata = self.results[v].to_numpy()
                 rdata[v] = np.min(vdata, axis=1)
             elif op == "max":
-                vdata = self.results[v].to_numpy()
                 rdata[v] = np.max(vdata, axis=1)
             else:
                 raise KeyError(
-                    f"Unknown operation '{op}' for variable '{v}'. Please choose: sum, mean, min, max"
+                    f"Unknown operation '{op}' for variable '{v}'. Please choose: weights, mean_no_weights, sum, min, max"
                 )
 
         data = pd.DataFrame(index=states, data=rdata)
@@ -204,29 +246,32 @@ class FarmResultsEval(Output):
         rdata = {}
         for v, op in turbines_op.items():
             vdata = sdata[v].to_numpy()
-            if op == "mean":
-                if states_op[v] == "mean":
+            nns = np.sum(np.isnan(vdata))
+            assert nns == 0, (
+                f"Found {nns} nan values for variable '{v}' of shape {vdata.shape}"
+            )
+
+            if op == "weights":
+                if states_op[v] == "weights":
                     rdata[v] = self.weinsum("", v)
                 else:
-                    vdata = sdata[v].to_numpy()
                     rdata[v] = self.weinsum("", vdata[None, :])
+            elif op == "mean_no_weights":
+                rdata[v] = np.sum(vdata)
             elif op == "sum":
-                vdata = sdata[v].to_numpy()
                 rdata[v] = np.sum(vdata)
             elif op == "min":
-                vdata = sdata[v].to_numpy()
                 rdata[v] = np.min(vdata)
             elif op == "max":
-                vdata = sdata[v].to_numpy()
                 rdata[v] = np.max(vdata)
             else:
                 raise KeyError(
-                    f"Unknown operation '{op}' for variable '{v}'. Please choose: sum, mean, min, max"
+                    f"Unknown operation '{op}' for variable '{v}'. Please choose: sum, mean, min, max, weights"
                 )
 
         return rdata
 
-    def calc_states_mean(self, vars):
+    def calc_states_mean(self, vars, use_weights=True):
         """
         Calculates the mean wrt states.
 
@@ -234,6 +279,8 @@ class FarmResultsEval(Output):
         ----------
         vars: list of str
             The variables
+        use_weights: bool
+            Flag for using states weights for the mean
 
         Returns
         -------
@@ -241,9 +288,10 @@ class FarmResultsEval(Output):
             The results per turbine
 
         """
+        r = "weights" if use_weights else "mean_no_weights"
         if isinstance(vars, str):
-            return self.reduce_states({vars: "mean"})
-        return self.reduce_states({v: "mean" for v in vars})
+            return self.reduce_states({vars: r})
+        return self.reduce_states({v: r for v in vars})
 
     def calc_states_sum(self, vars):
         """
@@ -290,7 +338,7 @@ class FarmResultsEval(Output):
             The results per state
 
         """
-        return self.reduce_turbines({v: "mean" for v in vars})
+        return self.reduce_turbines({v: "mean_no_weights" for v in vars})
 
     def calc_turbine_sum(self, vars):
         """
@@ -324,7 +372,7 @@ class FarmResultsEval(Output):
             The fully contracted results
 
         """
-        op = {v: "mean" for v in vars}
+        op = {v: "weights" for v in vars}
         return self.reduce_all(states_op=op, turbines_op=op)
 
     def calc_farm_sum(self, vars):
@@ -361,7 +409,7 @@ class FarmResultsEval(Output):
 
         """
         v = FV.P if not ambient else FV.AMB_P
-        cdata = self.reduce_all(states_op={v: "mean"}, turbines_op={v: "sum"})
+        cdata = self.reduce_all(states_op={v: "weights"}, turbines_op={v: "sum"})
         return cdata[v]
 
     def calc_turbine_yield(
@@ -381,7 +429,7 @@ class FarmResultsEval(Output):
         algo: foxes.core.Algorithm, optional
             The algorithm, for P_nominal lookup
         annual: bool, optional
-            Flag for returing annual results, by default False
+            Flag for returning annual results, by default False
         ambient: bool, optional
             Flag for ambient power, by default False
         hours: int, optional
@@ -409,7 +457,7 @@ class FarmResultsEval(Output):
         if algo is not None and P_unit_W is None:
             P_unit_W = np.array(
                 [FC.P_UNITS[t.P_unit] for t in algo.farm_controller.turbine_types],
-                dtype=FC.DTYPE,
+                dtype=config.dtype_double,
             )[:, None]
         elif algo is None and P_unit_W is not None:
             pass
@@ -426,11 +474,11 @@ class FarmResultsEval(Output):
             duration = times[-1] - times[0] + delta_t
             duration_seconds = np.int64(duration.astype(np.int64) / 1e9)
             duration_hours = duration_seconds / 3600
-        elif hours is None and annual == True:
+        elif hours is None and annual:
             duration_hours = 8760
         elif hours is None:
             raise ValueError(
-                f"Expecting parameter 'hours' for non-timeseries data, or 'annual=True'"
+                "Expecting parameter 'hours' for non-timeseries data, or 'annual=True'"
             )
         else:
             duration_hours = hours
@@ -472,10 +520,10 @@ class FarmResultsEval(Output):
         if algo is not None and P_nom is None:
             P_nom = np.array(
                 [t.P_nominal for t in algo.farm_controller.turbine_types],
-                dtype=FC.DTYPE,
+                dtype=config.dtype_double,
             )
         elif algo is None and P_nom is not None:
-            P_nom = np.array(P_nom, dtype=FC.DTYPE)
+            P_nom = np.array(P_nom, dtype=config.dtype_double)
         else:
             raise KeyError("Expecting either 'algo' or 'P_nom'")
 
@@ -535,10 +583,12 @@ class FarmResultsEval(Output):
             The verbosity level, 0 = silent
 
         """
-        P = self.results[FV.P]
-        P0 = self.results[FV.AMB_P] + 1e-14
-        self.results[FV.EFF] = P / P0  # add to farm results
-        if verbosity:
+        P = self.results[FV.P].to_numpy()
+        P0 = np.maximum(self.results[FV.AMB_P].to_numpy(), 1e-12)
+        eff = np.minimum(P / P0, 1)
+        eff[P < 1e-10] = 0
+        self.results[FV.EFF] = (self.results[FV.AMB_P].dims, eff)
+        if verbosity > 0:
             print("Efficiency added to farm results")
 
     def calc_farm_efficiency(self):
@@ -552,8 +602,8 @@ class FarmResultsEval(Output):
 
         """
         P = self.calc_mean_farm_power()
-        P0 = self.calc_mean_farm_power(ambient=True) + 1e-14
-        return P / P0
+        P0 = np.maximum(self.calc_mean_farm_power(ambient=True), 1e-14)
+        return np.minimum(P / P0, 1)
 
     def gen_stdata(
         self,
@@ -607,7 +657,7 @@ class FarmResultsEval(Output):
         else:
             hax = ax
 
-        hax.set_xlabel(f"State")
+        hax.set_xlabel("State")
         hax.set_ylabel(variable)
         cc = cycler(color="bgrcmyk")
 
