@@ -1,10 +1,10 @@
 import numpy as np
 
-from foxes.output import Output
 from foxes.utils import write_nc
-import foxes.constants as FC
 import foxes.variables as FV
+import foxes.constants as FC
 
+from .output import Output
 from . import grids
 
 
@@ -18,23 +18,20 @@ class SliceData(Output):
         The algorithm for point calculation
     farm_results: xarray.Dataset
         The farm results
-    runner: foxes.utils.runners.Runner, optional
-        The runner
     verbosity_delta: int
         Verbosity threshold for printing calculation info
-            
+
     :group: output
 
     """
 
     def __init__(
-        self, 
-        algo, 
-        farm_results, 
-        runner=None, 
+        self,
+        algo,
+        farm_results,
         verbosity_delta=1,
         **kwargs,
-        ):
+    ):
         """
         Constructor.
 
@@ -44,8 +41,6 @@ class SliceData(Output):
             The algorithm for point calculation
         farm_results: xarray.Dataset
             The farm results
-        runner: foxes.utils.runners.Runner, optional
-            The runner
         verbosity_delta: int
             Verbosity threshold for printing calculation info
         kwargs: dict, optional
@@ -55,7 +50,6 @@ class SliceData(Output):
         super().__init__(**kwargs)
         self.algo = algo
         self.fres = farm_results
-        self.runner = runner
         self.verbosity_delta = verbosity_delta
 
     def _data_mod(
@@ -129,9 +123,6 @@ class SliceData(Output):
         label_map,
         vmin,
         vmax,
-        states_sel,
-        states_isel,
-        weight_turbine,
         to_file,
         write_pars,
         ret_states,
@@ -144,9 +135,7 @@ class SliceData(Output):
             algo=self.algo,
             farm_results=self.fres,
             g_pts=g_pts,
-            sel={FC.STATE: states_sel} if states_sel is not None else None,
-            isel={FC.STATE: states_isel} if states_isel is not None else None,
-            verbosity=verbosity-self.verbosity_delta,
+            verbosity=verbosity - self.verbosity_delta,
             **kwargs,
         )
         states = point_results[FC.STATE].to_numpy()
@@ -157,12 +146,22 @@ class SliceData(Output):
         del g_pts
 
         # take mean over states:
-        weights = self.fres[FV.WEIGHT][:, weight_turbine].to_numpy()
-        data = {
-            v: np.einsum("s,sp->p", weights, point_results[v].to_numpy())
-            for v in variables
-        }
-        del point_results
+        weights = point_results[FV.WEIGHT].to_numpy()
+        if point_results[FV.WEIGHT].dims == (FC.STATE,):
+            data = {
+                v: np.einsum("s,sp->p", weights, point_results[v].to_numpy())
+                for v in variables
+            }
+        elif point_results[FV.WEIGHT].dims == (FC.STATE, FC.POINT):
+            data = {
+                v: np.einsum("sp,sp->p", weights, point_results[v].to_numpy())
+                for v in variables
+            }
+        else:
+            raise ValueError(
+                f"Wrong dimensions for '{FV.WEIGHT}': Expecting {(FC.STATE,)} or {(FC.STATE, FC.POINT)}, got {point_results[FV.WEIGHT].dims}"
+            )
+        del point_results, weights
 
         # apply data modification:
         a_pos, b_pos, c_pos, data = self._data_mod(
@@ -197,7 +196,8 @@ class SliceData(Output):
 
     def get_mean_data_xy(
         self,
-        resolution,
+        resolution=None,
+        n_img_points=None,
         variables=None,
         data_format="xarray",
         xmin=None,
@@ -216,7 +216,6 @@ class SliceData(Output):
         vmax={},
         states_sel=None,
         states_isel=None,
-        weight_turbine=0,
         to_file=None,
         write_pars={},
         ret_states=False,
@@ -229,8 +228,10 @@ class SliceData(Output):
 
         Parameters
         ----------
-        resolution: float
+        resolution: float, optional
             The resolution in m
+        n_img_points: tuple of int, optional
+            The number of image points (n, m) in the two directions
         variables: list of str, optional
             The variables, or None for all
         data_format: str
@@ -267,8 +268,6 @@ class SliceData(Output):
             Reduce to selected states
         states_isel: list, optional
             Reduce to the selected states indices
-        weight_turbine: int, optional
-            Index of the turbine from which to take the weight
         to_file: str, optional
             Write data to this file name
         write_pars: dict
@@ -294,16 +293,19 @@ class SliceData(Output):
 
         """
         gdata = grids.get_grid_xy(
-            self.fres, 
-            resolution, 
-            xmin, 
-            ymin, 
-            xmax, 
-            ymax, 
-            z, 
-            xspace, 
-            yspace, 
-            verbosity-self.verbosity_delta,
+            farm_results=self.fres,
+            resolution=resolution,
+            n_img_points=n_img_points,
+            xmin=xmin,
+            ymin=ymin,
+            xmax=xmax,
+            ymax=ymax,
+            z=z,
+            xspace=xspace,
+            yspace=yspace,
+            states_sel=states_sel,
+            states_isel=states_isel,
+            verbosity=verbosity - self.verbosity_delta,
         )
 
         data = self._calc_mean_data(
@@ -318,13 +320,12 @@ class SliceData(Output):
             label_map,
             vmin,
             vmax,
-            states_sel,
-            states_isel,
-            weight_turbine,
             to_file,
             write_pars,
             ret_states,
             verbosity,
+            states_sel=states_sel,
+            states_isel=states_isel,
             **kwargs,
         )
 
@@ -335,7 +336,8 @@ class SliceData(Output):
 
     def get_mean_data_xz(
         self,
-        resolution,
+        resolution=None,
+        n_img_points=None,
         variables=None,
         data_format="xarray",
         x_direction=270,
@@ -355,7 +357,6 @@ class SliceData(Output):
         vmax={},
         states_sel=None,
         states_isel=None,
-        weight_turbine=0,
         to_file=None,
         write_pars={},
         ret_states=False,
@@ -368,8 +369,10 @@ class SliceData(Output):
 
         Parameters
         ----------
-        resolution: float
+        resolution: float, optional
             The resolution in m
+        n_img_points: tuple of int, optional
+            The number of image points (n, m) in the two directions
         variables: list of str, optional
             The variables, or None for all
         data_format: str
@@ -408,8 +411,6 @@ class SliceData(Output):
             Reduce to selected states
         states_isel: list, optional
             Reduce to the selected states indices
-        weight_turbine: int, optional
-            Index of the turbine from which to take the weight
         to_file: str, optional
             Write data to this file name
         write_pars: dict
@@ -435,17 +436,20 @@ class SliceData(Output):
 
         """
         gdata = grids.get_grid_xz(
-            self.fres,
-            resolution,
-            x_direction,
-            xmin,
-            zmin,
-            xmax,
-            zmax,
-            y,
-            xspace,
-            zspace,
-            verbosity-self.verbosity_delta,
+            farm_results=self.fres,
+            resolution=resolution,
+            n_img_points=n_img_points,
+            x_direction=x_direction,
+            xmin=xmin,
+            zmin=zmin,
+            xmax=xmax,
+            zmax=zmax,
+            y=y,
+            xspace=xspace,
+            zspace=zspace,
+            states_sel=states_sel,
+            states_isel=states_isel,
+            verbosity=verbosity - self.verbosity_delta,
         )
         gdatb = (gdata[0], gdata[2], gdata[1], gdata[3])
 
@@ -461,13 +465,12 @@ class SliceData(Output):
             label_map,
             vmin,
             vmax,
-            states_sel,
-            states_isel,
-            weight_turbine,
             to_file,
             write_pars,
             ret_states,
             verbosity,
+            states_sel=states_sel,
+            states_isel=states_isel,
             **kwargs,
         )
 
@@ -478,7 +481,8 @@ class SliceData(Output):
 
     def get_mean_data_yz(
         self,
-        resolution,
+        resolution=None,
+        n_img_points=None,
         variables=None,
         data_format="xarray",
         x_direction=270,
@@ -498,7 +502,6 @@ class SliceData(Output):
         vmax={},
         states_sel=None,
         states_isel=None,
-        weight_turbine=0,
         to_file=None,
         write_pars={},
         ret_states=False,
@@ -511,8 +514,10 @@ class SliceData(Output):
 
         Parameters
         ----------
-        resolution: float
+        resolution: float, optional
             The resolution in m
+        n_img_points: tuple of int, optional
+            The number of image points (n, m) in the two directions
         variables: list of str, optional
             The variables, or None for all
         data_format: str
@@ -551,8 +556,6 @@ class SliceData(Output):
             Reduce to selected states
         states_isel: list, optional
             Reduce to the selected states indices
-        weight_turbine: int, optional
-            Index of the turbine from which to take the weight
         to_file: str, optional
             Write data to this file name
         write_pars: dict
@@ -578,17 +581,20 @@ class SliceData(Output):
 
         """
         gdata = grids.get_grid_yz(
-            self.fres,
-            resolution,
-            x_direction,
-            ymin,
-            zmin,
-            ymax,
-            zmax,
-            x,
-            yspace,
-            zspace,
-            verbosity-self.verbosity_delta,
+            farm_results=self.fres,
+            resolution=resolution,
+            n_img_points=n_img_points,
+            x_direction=x_direction,
+            ymin=ymin,
+            zmin=zmin,
+            ymax=ymax,
+            zmax=zmax,
+            x=x,
+            yspace=yspace,
+            zspace=zspace,
+            states_sel=states_sel,
+            states_isel=states_isel,
+            verbosity=verbosity - self.verbosity_delta,
         )
         gdatb = (gdata[1], gdata[2], gdata[0], gdata[3])
 
@@ -604,13 +610,12 @@ class SliceData(Output):
             label_map,
             vmin,
             vmax,
-            states_sel,
-            states_isel,
-            weight_turbine,
             to_file,
             write_pars,
             ret_states,
             verbosity,
+            states_sel=states_sel,
+            states_isel=states_isel,
             **kwargs,
         )
 
@@ -635,8 +640,6 @@ class SliceData(Output):
         label_map,
         vmin,
         vmax,
-        states_sel,
-        states_isel,
         to_file,
         write_pars,
         ret_states,
@@ -645,15 +648,11 @@ class SliceData(Output):
     ):
         """Helper function for states data calculation"""
         # calculate point results:
-        if states_sel is not None:
-            kwargs["sel"] = {FC.STATE: states_sel}
-        if states_isel is not None:
-            kwargs["isel"] = {FC.STATE: states_isel}
         point_results = grids.calc_point_results(
             algo=self.algo,
             farm_results=self.fres,
             g_pts=g_pts,
-            verbosity=verbosity-self.verbosity_delta,
+            verbosity=verbosity - self.verbosity_delta,
             **kwargs,
         )
         states = point_results[FC.STATE].to_numpy()
@@ -700,7 +699,8 @@ class SliceData(Output):
 
     def get_states_data_xy(
         self,
-        resolution,
+        resolution=None,
+        n_img_points=None,
         variables=None,
         data_format="xarray",
         xmin=None,
@@ -731,8 +731,10 @@ class SliceData(Output):
 
         Parameters
         ----------
-        resolution: float
+        resolution: float, optional
             The resolution in m
+        n_img_points: tuple of int, optional
+            The number of image points (n, m) in the two directions
         variables: list of str, optional
             The variables, or None for all
         data_format: str
@@ -794,16 +796,19 @@ class SliceData(Output):
 
         """
         gdata = grids.get_grid_xy(
-            self.fres, 
-            resolution, 
-            xmin, 
-            ymin, 
-            xmax, 
-            ymax, 
-            z, 
-            xspace, 
-            yspace, 
-            verbosity-self.verbosity_delta,
+            farm_results=self.fres,
+            resolution=resolution,
+            n_img_points=n_img_points,
+            xmin=xmin,
+            ymin=ymin,
+            xmax=xmax,
+            ymax=ymax,
+            z=z,
+            xspace=xspace,
+            yspace=yspace,
+            states_sel=states_sel,
+            states_isel=states_isel,
+            verbosity=verbosity - self.verbosity_delta,
         )
 
         data = self._calc_states_data(
@@ -818,12 +823,12 @@ class SliceData(Output):
             label_map,
             vmin,
             vmax,
-            states_sel,
-            states_isel,
             to_file,
             write_pars,
             ret_states,
             verbosity,
+            states_sel=states_sel,
+            states_isel=states_isel,
             **kwargs,
         )
 
@@ -834,7 +839,8 @@ class SliceData(Output):
 
     def get_states_data_xz(
         self,
-        resolution,
+        resolution=None,
+        n_img_points=None,
         variables=None,
         data_format="xarray",
         x_direction=270,
@@ -866,8 +872,10 @@ class SliceData(Output):
 
         Parameters
         ----------
-        resolution: float
+        resolution: float, optional
             The resolution in m
+        n_img_points: tuple of int, optional
+            The number of image points (n, m) in the two directions
         variables: list of str, optional
             The variables, or None for all
         data_format: str
@@ -931,17 +939,20 @@ class SliceData(Output):
 
         """
         gdata = grids.get_grid_xz(
-            self.fres,
-            resolution,
-            x_direction,
-            xmin,
-            zmin,
-            xmax,
-            zmax,
-            y,
-            xspace,
-            zspace,
-            verbosity-self.verbosity_delta,
+            farm_results=self.fres,
+            resolution=resolution,
+            n_img_points=n_img_points,
+            x_direction=x_direction,
+            xmin=xmin,
+            zmin=zmin,
+            xmax=xmax,
+            zmax=zmax,
+            y=y,
+            xspace=xspace,
+            zspace=zspace,
+            states_sel=states_sel,
+            states_isel=states_isel,
+            verbosity=verbosity - self.verbosity_delta,
         )
         gdatb = (gdata[0], gdata[2], gdata[1], gdata[3])
 
@@ -957,12 +968,12 @@ class SliceData(Output):
             label_map,
             vmin,
             vmax,
-            states_sel,
-            states_isel,
             to_file,
             write_pars,
             ret_states,
             verbosity,
+            states_sel=states_sel,
+            states_isel=states_isel,
             **kwargs,
         )
 
@@ -973,7 +984,8 @@ class SliceData(Output):
 
     def get_states_data_yz(
         self,
-        resolution,
+        resolution=None,
+        n_img_points=None,
         variables=None,
         data_format="xarray",
         x_direction=270,
@@ -1005,8 +1017,10 @@ class SliceData(Output):
 
         Parameters
         ----------
-        resolution: float
+        resolution: float, optional
             The resolution in m
+        n_img_points: tuple of int, optional
+            The number of image points (n, m) in the two directions
         variables: list of str, optional
             The variables, or None for all
         data_format: str
@@ -1070,17 +1084,20 @@ class SliceData(Output):
 
         """
         gdata = grids.get_grid_yz(
-            self.fres,
-            resolution,
-            x_direction,
-            ymin,
-            zmin,
-            ymax,
-            zmax,
-            x,
-            yspace,
-            zspace,
-            verbosity-self.verbosity_delta,
+            farm_results=self.fres,
+            resolution=resolution,
+            n_img_points=n_img_points,
+            x_direction=x_direction,
+            ymin=ymin,
+            zmin=zmin,
+            ymax=ymax,
+            zmax=zmax,
+            x=x,
+            yspace=yspace,
+            zspace=zspace,
+            states_sel=states_sel,
+            states_isel=states_isel,
+            verbosity=verbosity - self.verbosity_delta,
         )
         gdatb = (gdata[1], gdata[2], gdata[0], gdata[3])
 
@@ -1096,12 +1113,12 @@ class SliceData(Output):
             label_map,
             vmin,
             vmax,
-            states_sel,
-            states_isel,
             to_file,
             write_pars,
             ret_states,
             verbosity,
+            states_sel=states_sel,
+            states_isel=states_isel,
             **kwargs,
         )
 
